@@ -13,7 +13,6 @@ For more details see `this discussion`_ on github.
 .. _this discussion: https://github.com/pydata/xarray/issues/1397#issuecomment-299190567
 """
 
-
 import cartopy.crs as ccrs
 import matplotlib.pyplot as plt
 
@@ -30,7 +29,7 @@ p = air.plot(
     transform=ccrs.PlateCarree(),  # the data's projection
     col="time",
     col_wrap=1,  # multiplot settings
-    aspect=ds.dims["lon"] / ds.dims["lat"],  # for a sensible figsize
+    aspect=ds.sizes["lon"] / ds.sizes["lat"],  # for a sensible figsize
     subplot_kws={"projection": map_proj},  # the plot's projection
 )
 
