@@ -8,6 +8,7 @@ import pandas as pd
 
 from xarray.core import dtypes, utils
 from xarray.core.alignment import align, reindex_variables
+from xarray.core.coordinates import Coordinates
 from xarray.core.duck_array_ops import lazy_array_equiv
 from xarray.core.indexes import Index, PandasIndex
 from xarray.core.merge import (
@@ -31,10 +32,11 @@ if TYPE_CHECKING:
     T_DataVars = Union[ConcatOptions, Iterable[Hashable]]
 
 
+# TODO: replace dim: Any by 1D array_likes
 @overload
 def concat(
     objs: Iterable[T_Dataset],
-    dim: Hashable | T_Variable | T_DataArray | pd.Index,
+    dim: Hashable | T_Variable | T_DataArray | pd.Index | Any,
     data_vars: T_DataVars = "all",
     coords: ConcatOptions | list[Hashable] = "different",
     compat: CompatOptions = "equals",
@@ -42,13 +44,14 @@ def concat(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
+    create_index_for_new_dim: bool = True,
 ) -> T_Dataset: ...
 
 
 @overload
 def concat(
     objs: Iterable[T_DataArray],
-    dim: Hashable | T_Variable | T_DataArray | pd.Index,
+    dim: Hashable | T_Variable | T_DataArray | pd.Index | Any,
     data_vars: T_DataVars = "all",
     coords: ConcatOptions | list[Hashable] = "different",
     compat: CompatOptions = "equals",
@@ -56,6 +59,7 @@ def concat(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
+    create_index_for_new_dim: bool = True,
 ) -> T_DataArray: ...
 
 
@@ -69,6 +73,7 @@ def concat(
     fill_value=dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
+    create_index_for_new_dim: bool = True,
 ):
     """Concatenate xarray objects along a new or existing dimension.
 
@@ -162,6 +167,8 @@ def concat(
 
         If a callable, it must expect a sequence of ``attrs`` dicts and a context object
         as its only parameters.
+    create_index_for_new_dim : bool, default: True
+        Whether to create a new ``PandasIndex`` object when the objects being concatenated contain scalar variables named ``dim``.
 
     Returns
     -------
@@ -217,6 +224,25 @@ def concat(
         x        (new_dim) <U1 8B 'a' 'b'
       * y        (y) int64 24B 10 20 30
       * new_dim  (new_dim) int64 16B -90 -100
+
+    # Concatenate a scalar variable along a new dimension of the same name with and without creating a new index
+
+    >>> ds = xr.Dataset(coords={"x": 0})
+    >>> xr.concat([ds, ds], dim="x")
+    <xarray.Dataset> Size: 16B
+    Dimensions:  (x: 2)
+    Coordinates:
+      * x        (x) int64 16B 0 0
+    Data variables:
+        *empty*
+
+    >>> xr.concat([ds, ds], dim="x").indexes
+    Indexes:
+        x        Index([0, 0], dtype='int64', name='x')
+
+    >>> xr.concat([ds, ds], dim="x", create_index_for_new_dim=False).indexes
+    Indexes:
+        *empty*
     """
     # TODO: add ignore_index arguments copied from pandas.concat
     # TODO: support concatenating scalar coordinates even if the concatenated
@@ -226,10 +252,10 @@ def concat(
 
     try:
         first_obj, objs = utils.peek_at(objs)
-    except StopIteration:
-        raise ValueError("must supply at least one object to concatenate")
+    except StopIteration as err:
+        raise ValueError("must supply at least one object to concatenate") from err
 
-    if compat not in _VALID_COMPAT:
+    if compat not in set(_VALID_COMPAT) - {"minimal"}:
         raise ValueError(
             f"compat={compat!r} invalid: must be 'broadcast_equals', 'equals', 'identical', 'no_conflicts' or 'override'"
         )
@@ -245,6 +271,7 @@ def concat(
             fill_value=fill_value,
             join=join,
             combine_attrs=combine_attrs,
+            create_index_for_new_dim=create_index_for_new_dim,
         )
     elif isinstance(first_obj, Dataset):
         return _dataset_concat(
@@ -257,6 +284,7 @@ def concat(
             fill_value=fill_value,
             join=join,
             combine_attrs=combine_attrs,
+            create_index_for_new_dim=create_index_for_new_dim,
         )
     else:
         raise TypeError(
@@ -276,11 +304,11 @@ def _calc_concat_dim_index(
 
     dim: Hashable | None
 
-    if isinstance(dim_or_data, str):
+    if utils.hashable(dim_or_data):
         dim = dim_or_data
         index = None
     else:
-        if not isinstance(dim_or_data, (DataArray, Variable)):
+        if not isinstance(dim_or_data, DataArray | Variable):
             dim = getattr(dim_or_data, "name", None)
             if dim is None:
                 dim = "concat_dim"
@@ -372,7 +400,9 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars: T_DataVars, coords, c
                                     equals[k] = False
                                     # computed variables are not to be re-computed
                                     # again in the future
-                                    for ds, v in zip(datasets[1:], computed):
+                                    for ds, v in zip(
+                                        datasets[1:], computed, strict=False
+                                    ):
                                         ds.variables[k].data = v.data
                                     break
                             else:
@@ -439,7 +469,7 @@ def _parse_datasets(
             if dim in dims:
                 continue
 
-            if dim not in dim_coords:
+            if dim in ds.coords and dim not in dim_coords:
                 dim_coords[dim] = ds.coords[dim].variable
         dims = dims | set(ds.dims)
 
@@ -447,7 +477,7 @@ def _parse_datasets(
 
 
 def _dataset_concat(
-    datasets: list[T_Dataset],
+    datasets: Iterable[T_Dataset],
     dim: str | T_Variable | T_DataArray | pd.Index,
     data_vars: T_DataVars,
     coords: str | list[str],
@@ -456,6 +486,7 @@ def _dataset_concat(
     fill_value: Any = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
+    create_index_for_new_dim: bool = True,
 ) -> T_Dataset:
     """
     Concatenate a sequence of datasets along a new or existing dimension
@@ -477,19 +508,20 @@ def _dataset_concat(
     else:
         dim_var = None
 
-    dim, index = _calc_concat_dim_index(dim)
+    dim_name, index = _calc_concat_dim_index(dim)
 
     # Make sure we're working on a copy (we'll be loading variables)
     datasets = [ds.copy() for ds in datasets]
     datasets = list(
-        align(*datasets, join=join, copy=False, exclude=[dim], fill_value=fill_value)
+        align(
+            *datasets, join=join, copy=False, exclude=[dim_name], fill_value=fill_value
+        )
     )
 
     dim_coords, dims_sizes, coord_names, data_names, vars_order = _parse_datasets(
         datasets
     )
     dim_names = set(dim_coords)
-    unlabeled_dims = dim_names - coord_names
 
     both_data_and_coords = coord_names & data_names
     if both_data_and_coords:
@@ -497,20 +529,25 @@ def _dataset_concat(
             f"{both_data_and_coords!r} is a coordinate in some datasets but not others."
         )
     # we don't want the concat dimension in the result dataset yet
-    dim_coords.pop(dim, None)
-    dims_sizes.pop(dim, None)
+    dim_coords.pop(dim_name, None)
+    dims_sizes.pop(dim_name, None)
 
     # case where concat dimension is a coordinate or data_var but not a dimension
-    if (dim in coord_names or dim in data_names) and dim not in dim_names:
-        datasets = [ds.expand_dims(dim) for ds in datasets]
+    if (
+        dim_name in coord_names or dim_name in data_names
+    ) and dim_name not in dim_names:
+        datasets = [
+            ds.expand_dims(dim_name, create_index_for_new_dim=create_index_for_new_dim)
+            for ds in datasets
+        ]
 
     # determine which variables to concatenate
     concat_over, equals, concat_dim_lengths = _calc_concat_over(
-        datasets, dim, dim_names, data_vars, coords, compat
+        datasets, dim_name, dim_names, data_vars, coords, compat
     )
 
     # determine which variables to merge, and then merge them according to compat
-    variables_to_merge = (coord_names | data_names) - concat_over - unlabeled_dims
+    variables_to_merge = (coord_names | data_names) - concat_over
 
     result_vars = {}
     result_indexes = {}
@@ -546,9 +583,9 @@ def _dataset_concat(
         # dimensions and the same shape for all of them except along the
         # concat dimension
         common_dims = tuple(utils.OrderedSet(d for v in vars for d in v.dims))
-        if dim not in common_dims:
-            common_dims = (dim,) + common_dims
-        for var, dim_len in zip(vars, concat_dim_lengths):
+        if dim_name not in common_dims:
+            common_dims = (dim_name,) + common_dims
+        for var, dim_len in zip(vars, concat_dim_lengths, strict=True):
             if var.dims != common_dims:
                 common_shape = tuple(dims_sizes.get(d, dim_len) for d in common_dims)
                 var = var.set_dims(common_dims, common_shape)
@@ -563,11 +600,12 @@ def _dataset_concat(
         for ds in datasets:
             if name in ds._indexes:
                 yield ds._indexes[name]
-            elif name == dim:
+            elif name == dim_name:
                 var = ds._variables[name]
                 if not var.dims:
-                    data = var.set_dims(dim).values
-                    yield PandasIndex(data, dim, coord_dtype=var.dtype)
+                    data = var.set_dims(dim_name).values
+                    if create_index_for_new_dim:
+                        yield PandasIndex(data, dim_name, coord_dtype=var.dtype)
 
     # create concatenation index, needed for later reindexing
     file_start_indexes = np.append(0, np.cumsum(concat_dim_lengths))
@@ -613,7 +651,7 @@ def _dataset_concat(
                         f"{name!r} must have either an index or no index in all datasets, "
                         f"found {len(indexes)}/{len(datasets)} datasets with an index."
                     )
-                combined_idx = indexes[0].concat(indexes, dim, positions)
+                combined_idx = indexes[0].concat(indexes, dim_name, positions)
                 if name in datasets[0]._indexes:
                     idx_vars = datasets[0].xindexes.get_all_coords(name)
                 else:
@@ -629,14 +667,14 @@ def _dataset_concat(
                     result_vars[k] = v
             else:
                 combined_var = concat_vars(
-                    vars, dim, positions, combine_attrs=combine_attrs
+                    vars, dim_name, positions, combine_attrs=combine_attrs
                 )
                 # reindex if variable is not present in all datasets
                 if len(variable_index) < concat_index_size:
                     combined_var = reindex_variables(
                         variables={name: combined_var},
                         dim_pos_indexers={
-                            dim: pd.Index(variable_index).get_indexer(concat_index)
+                            dim_name: pd.Index(variable_index).get_indexer(concat_index)
                         },
                         fill_value=fill_value,
                     )[name]
@@ -646,29 +684,33 @@ def _dataset_concat(
             # preserves original variable order
             result_vars[name] = result_vars.pop(name)
 
-    result = type(datasets[0])(result_vars, attrs=result_attrs)
-
-    absent_coord_names = coord_names - set(result.variables)
+    absent_coord_names = coord_names - set(result_vars)
     if absent_coord_names:
         raise ValueError(
             f"Variables {absent_coord_names!r} are coordinates in some datasets but not others."
         )
-    result = result.set_coords(coord_names)
-    result.encoding = result_encoding
 
-    result = result.drop_vars(unlabeled_dims, errors="ignore")
+    result_data_vars = {}
+    coord_vars = {}
+    for name, result_var in result_vars.items():
+        if name in coord_names:
+            coord_vars[name] = result_var
+        else:
+            result_data_vars[name] = result_var
 
     if index is not None:
-        # add concat index / coordinate last to ensure that its in the final Dataset
         if dim_var is not None:
-            index_vars = index.create_variables({dim: dim_var})
+            index_vars = index.create_variables({dim_name: dim_var})
         else:
             index_vars = index.create_variables()
-        result[dim] = index_vars[dim]
-        result_indexes[dim] = index
 
-    # TODO: add indexes at Dataset creation (when it is supported)
-    result = result._overwrite_indexes(result_indexes)
+        coord_vars[dim_name] = index_vars[dim_name]
+        result_indexes[dim_name] = index
+
+    coords_obj = Coordinates(coord_vars, indexes=result_indexes)
+
+    result = type(datasets[0])(result_data_vars, coords=coords_obj, attrs=result_attrs)
+    result.encoding = result_encoding
 
     return result
 
@@ -683,6 +725,7 @@ def _dataarray_concat(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
+    create_index_for_new_dim: bool = True,
 ) -> T_DataArray:
     from xarray.core.dataarray import DataArray
 
@@ -719,6 +762,7 @@ def _dataarray_concat(
         fill_value=fill_value,
         join=join,
         combine_attrs=combine_attrs,
+        create_index_for_new_dim=create_index_for_new_dim,
     )
 
     merged_attrs = merge_attrs([da.attrs for da in arrays], combine_attrs)
