@@ -1,25 +1,36 @@
-"""String formatting routines for __repr__.
-"""
+"""String formatting routines for __repr__."""
+
 from __future__ import annotations
 
 import contextlib
 import functools
 import math
-from collections import defaultdict
-from collections.abc import Collection, Hashable
+from collections import ChainMap, defaultdict
+from collections.abc import Collection, Hashable, Mapping, Sequence
 from datetime import datetime, timedelta
 from itertools import chain, zip_longest
 from reprlib import recursive_repr
+from textwrap import indent
+from typing import TYPE_CHECKING, Any
 
 import numpy as np
 import pandas as pd
 from pandas.errors import OutOfBoundsDatetime
 
-from xarray.core.duck_array_ops import array_equiv
-from xarray.core.indexing import ExplicitlyIndexed, MemoryCachedArray
+from xarray.core.datatree_render import RenderDataTree
+from xarray.core.duck_array_ops import array_equiv, astype
+from xarray.core.indexing import MemoryCachedArray
 from xarray.core.options import OPTIONS, _get_boolean_with_default
-from xarray.core.pycompat import array_type
+from xarray.core.treenode import group_subtrees
 from xarray.core.utils import is_duck_array
+from xarray.namedarray.pycompat import array_type, to_duck_array, to_numpy
+
+if TYPE_CHECKING:
+    from xarray.core.coordinates import AbstractCoordinates
+    from xarray.core.datatree import DataTree
+    from xarray.core.variable import Variable
+
+UNITS = ("B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
 
 
 def pretty_print(x, numchars: int):
@@ -64,6 +75,8 @@ def first_n_items(array, n_desired):
     # might not be a numpy.ndarray. Moreover, access to elements of the array
     # could be very expensive (e.g. if it's only available over DAP), so go out
     # of our way to get them in a single call to __getitem__ using only slices.
+    from xarray.core.variable import Variable
+
     if n_desired < 1:
         raise ValueError("must request at least one item")
 
@@ -74,7 +87,14 @@ def first_n_items(array, n_desired):
     if n_desired < array.size:
         indexer = _get_indexer_at_least_n_items(array.shape, n_desired, from_end=False)
         array = array[indexer]
-    return np.asarray(array).flat[:n_desired]
+
+    # We pass variable objects in to handle indexing
+    # with indexer above. It would not work with our
+    # lazy indexing classes at the moment, so we cannot
+    # pass Variable._data
+    if isinstance(array, Variable):
+        array = array._data
+    return np.ravel(to_duck_array(array))[:n_desired]
 
 
 def last_n_items(array, n_desired):
@@ -83,13 +103,22 @@ def last_n_items(array, n_desired):
     # might not be a numpy.ndarray. Moreover, access to elements of the array
     # could be very expensive (e.g. if it's only available over DAP), so go out
     # of our way to get them in a single call to __getitem__ using only slices.
+    from xarray.core.variable import Variable
+
     if (n_desired == 0) or (array.size == 0):
         return []
 
     if n_desired < array.size:
         indexer = _get_indexer_at_least_n_items(array.shape, n_desired, from_end=True)
         array = array[indexer]
-    return np.asarray(array).flat[-n_desired:]
+
+    # We pass variable objects in to handle indexing
+    # with indexer above. It would not work with our
+    # lazy indexing classes at the moment, so we cannot
+    # pass Variable._data
+    if isinstance(array, Variable):
+        array = array._data
+    return np.ravel(to_duck_array(array))[-n_desired:]
 
 
 def last_item(array):
@@ -99,7 +128,8 @@ def last_item(array):
         return []
 
     indexer = (slice(-1, None),) * array.ndim
-    return np.ravel(np.asarray(array[indexer])).tolist()
+    # to_numpy since dask doesn't support tolist
+    return np.ravel(to_numpy(array[indexer])).tolist()
 
 
 def calc_max_rows_first(max_rows: int) -> int:
@@ -151,11 +181,13 @@ def format_timedelta(t, timedelta_format=None):
 
 def format_item(x, timedelta_format=None, quote_strings=True):
     """Returns a succinct summary of an object as a string"""
-    if isinstance(x, (np.datetime64, datetime)):
+    if isinstance(x, np.datetime64 | datetime):
         return format_timestamp(x)
-    if isinstance(x, (np.timedelta64, timedelta)):
+    if isinstance(x, np.timedelta64 | timedelta):
         return format_timedelta(x, timedelta_format=timedelta_format)
-    elif isinstance(x, (str, bytes)):
+    elif isinstance(x, str | bytes):
+        if hasattr(x, "dtype"):
+            x = x.item()
         return repr(x) if quote_strings else x
     elif hasattr(x, "dtype") and np.issubdtype(x.dtype, np.floating):
         return f"{x.item():.4}"
@@ -165,10 +197,10 @@ def format_item(x, timedelta_format=None, quote_strings=True):
 
 def format_items(x):
     """Returns a succinct summaries of all items in a sequence as strings"""
-    x = np.asarray(x)
+    x = to_duck_array(x)
     timedelta_format = "datetime"
     if np.issubdtype(x.dtype, np.timedelta64):
-        x = np.asarray(x, dtype="timedelta64[ns]")
+        x = astype(x, dtype="timedelta64[ns]")
         day_part = x[~pd.isnull(x)].astype("timedelta64[D]").astype("timedelta64[ns]")
         time_needed = x[~pd.isnull(x)] != day_part
         day_needed = day_part != np.timedelta64(0, "ns")
@@ -261,16 +293,14 @@ def inline_sparse_repr(array):
     """Similar to sparse.COO.__repr__, but without the redundant shape/dtype."""
     sparse_array_type = array_type("sparse")
     assert isinstance(array, sparse_array_type), array
-    return "<{}: nnz={:d}, fill_value={!s}>".format(
-        type(array).__name__, array.nnz, array.fill_value
-    )
+    return f"<{type(array).__name__}: nnz={array.nnz:d}, fill_value={array.fill_value}>"
 
 
 def inline_variable_array_repr(var, max_width):
     """Build a one-line summary of a variable's data."""
     if hasattr(var._data, "_repr_inline_"):
         return var._data._repr_inline_(max_width)
-    if var._in_memory:
+    if getattr(var, "_in_memory", False):
         return format_array_flat(var, max_width)
     dask_array_type = array_type("dask")
     if isinstance(var._data, dask_array_type):
@@ -286,7 +316,7 @@ def inline_variable_array_repr(var, max_width):
 
 def summarize_variable(
     name: Hashable,
-    var,
+    var: Variable,
     col_width: int,
     max_width: int | None = None,
     is_index: bool = False,
@@ -308,7 +338,9 @@ def summarize_variable(
         dims_str = "({}) ".format(", ".join(map(str, variable.dims)))
     else:
         dims_str = ""
-    front_str = f"{first_col}{dims_str}{variable.dtype} "
+
+    nbytes_str = f" {render_human_readable_nbytes(variable.nbytes)}"
+    front_str = f"{first_col}{dims_str}{variable.dtype}{nbytes_str} "
 
     values_width = max_width - len(front_str)
     values_str = inline_variable_array_repr(variable, values_width)
@@ -332,7 +364,7 @@ EMPTY_REPR = "    *empty*"
 
 
 def _calculate_col_width(col_items):
-    max_name_length = max(len(str(s)) for s in col_items) if col_items else 0
+    max_name_length = max((len(str(s)) for s in col_items), default=0)
     col_width = max(max_name_length, 7) + 6
     return col_width
 
@@ -398,7 +430,7 @@ attrs_repr = functools.partial(
 )
 
 
-def coords_repr(coords, col_width=None, max_rows=None):
+def coords_repr(coords: AbstractCoordinates, col_width=None, max_rows=None):
     if col_width is None:
         col_width = _calculate_col_width(coords)
     return _mapping_repr(
@@ -412,7 +444,22 @@ def coords_repr(coords, col_width=None, max_rows=None):
     )
 
 
-def inline_index_repr(index, max_width=None):
+def inherited_coords_repr(node: DataTree, col_width=None, max_rows=None):
+    coords = inherited_vars(node._coord_variables)
+    if col_width is None:
+        col_width = _calculate_col_width(coords)
+    return _mapping_repr(
+        coords,
+        title="Inherited coordinates",
+        summarizer=summarize_variable,
+        expand_option_name="display_expand_coords",
+        col_width=col_width,
+        indexes=node._indexes,
+        max_rows=max_rows,
+    )
+
+
+def inline_index_repr(index: pd.Index, max_width=None):
     if hasattr(index, "_repr_inline_"):
         repr_ = index._repr_inline_(max_width=max_width)
     else:
@@ -440,7 +487,7 @@ def summarize_index(
 
     preformatted = [
         pretty_print(f"  {prefix} {name}", col_width)
-        for prefix, name in zip(prefixes(len(names)), names)
+        for prefix, name in zip(prefixes(len(names)), names, strict=True)
     ]
 
     head, *tail = preformatted
@@ -464,12 +511,12 @@ def filter_nondefault_indexes(indexes, filter_indexes: bool):
     }
 
 
-def indexes_repr(indexes, max_rows: int | None = None) -> str:
+def indexes_repr(indexes, max_rows: int | None = None, title: str = "Indexes") -> str:
     col_width = _calculate_col_width(chain.from_iterable(indexes))
 
     return _mapping_repr(
         indexes,
-        "Indexes",
+        title,
         summarize_index,
         "display_expand_indexes",
         col_width=col_width,
@@ -537,8 +584,10 @@ def _element_formatter(
     return "".join(out)
 
 
-def dim_summary_limited(obj, col_width: int, max_rows: int | None = None) -> str:
-    elements = [f"{k}: {v}" for k, v in obj.sizes.items()]
+def dim_summary_limited(
+    sizes: Mapping[Any, int], col_width: int, max_rows: int | None = None
+) -> str:
+    elements = [f"{k}: {v}" for k, v in sizes.items()]
     return _element_formatter(elements, col_width, max_rows)
 
 
@@ -578,12 +627,9 @@ def limit_lines(string: str, *, limit: int):
 def short_array_repr(array):
     from xarray.core.common import AbstractArray
 
-    if isinstance(array, ExplicitlyIndexed):
-        array = array.get_duck_array()
-    elif isinstance(array, AbstractArray):
+    if isinstance(array, AbstractArray):
         array = array.data
-    if not is_duck_array(array):
-        array = np.asarray(array)
+    array = to_duck_array(array)
 
     # default to lower precision so a full (abbreviated) line can fit on
     # one line with the default display_width
@@ -610,7 +656,7 @@ def short_data_repr(array):
         return short_array_repr(array)
     elif is_duck_array(internal_data):
         return limit_lines(repr(array.data), limit=40)
-    elif array._in_memory:
+    elif getattr(array, "_in_memory", None):
         return short_array_repr(array)
     else:
         # internal xarray array type
@@ -645,12 +691,12 @@ def array_repr(arr):
         data_repr = inline_variable_array_repr(arr.variable, OPTIONS["display_width"])
 
     start = f"<xarray.{type(arr).__name__} {name_str}"
-    dims = dim_summary_limited(arr, col_width=len(start) + 1, max_rows=max_rows)
+    dims = dim_summary_limited(arr.sizes, col_width=len(start) + 1, max_rows=max_rows)
+    nbytes_str = render_human_readable_nbytes(arr.nbytes)
     summary = [
-        f"{start}({dims})>",
+        f"{start}({dims})> Size: {nbytes_str}",
         data_repr,
     ]
-
     if hasattr(arr, "coords"):
         if arr.coords:
             col_width = _calculate_col_width(arr.coords)
@@ -683,13 +729,16 @@ def array_repr(arr):
 
 @recursive_repr("<recursive Dataset>")
 def dataset_repr(ds):
-    summary = [f"<xarray.{type(ds).__name__}>"]
+    nbytes_str = render_human_readable_nbytes(ds.nbytes)
+    summary = [f"<xarray.{type(ds).__name__}> Size: {nbytes_str}"]
 
     col_width = _calculate_col_width(ds.variables)
     max_rows = OPTIONS["display_max_rows"]
 
     dims_start = pretty_print("Dimensions:", col_width)
-    dims_values = dim_summary_limited(ds, col_width=col_width + 1, max_rows=max_rows)
+    dims_values = dim_summary_limited(
+        ds.sizes, col_width=col_width + 1, max_rows=max_rows
+    )
     summary.append(f"{dims_start}({dims_values})")
 
     if ds.coords:
@@ -716,8 +765,38 @@ def dataset_repr(ds):
     return "\n".join(summary)
 
 
-def diff_dim_summary(a, b):
-    if a.dims != b.dims:
+def dims_and_coords_repr(ds) -> str:
+    """Partial Dataset repr for use inside DataTree inheritance errors."""
+    summary = []
+
+    col_width = _calculate_col_width(ds.coords)
+    max_rows = OPTIONS["display_max_rows"]
+
+    dims_start = pretty_print("Dimensions:", col_width)
+    dims_values = dim_summary_limited(
+        ds.sizes, col_width=col_width + 1, max_rows=max_rows
+    )
+    summary.append(f"{dims_start}({dims_values})")
+
+    if ds.coords:
+        summary.append(coords_repr(ds.coords, col_width=col_width, max_rows=max_rows))
+
+    unindexed_dims_str = unindexed_dims_repr(ds.dims, ds.coords, max_rows=max_rows)
+    if unindexed_dims_str:
+        summary.append(unindexed_dims_str)
+
+    return "\n".join(summary)
+
+
+def diff_name_summary(a, b) -> str:
+    if a.name != b.name:
+        return f"Differing names:\n    {a.name!r} != {b.name!r}"
+    else:
+        return ""
+
+
+def diff_dim_summary(a, b) -> str:
+    if a.sizes != b.sizes:
         return f"Differing dimensions:\n    ({dim_summary(a)}) != ({dim_summary(b)})"
     else:
         return ""
@@ -733,6 +812,12 @@ def _diff_mapping_repr(
     a_indexes=None,
     b_indexes=None,
 ):
+    def compare_attr(a, b):
+        if is_duck_array(a) or is_duck_array(b):
+            return array_equiv(a, b)
+        else:
+            return a == b
+
     def extra_items_repr(extra_keys, mapping, ab_side, kwargs):
         extra_repr = [
             summarizer(k, mapping[k], col_width, **kwargs[k]) for k in extra_keys
@@ -761,17 +846,15 @@ def _diff_mapping_repr(
         try:
             # compare xarray variable
             if not callable(compat):
-                compatible = getattr(a_mapping[k], compat)(b_mapping[k])
+                compatible = getattr(a_mapping[k].variable, compat)(
+                    b_mapping[k].variable
+                )
             else:
-                compatible = compat(a_mapping[k], b_mapping[k])
+                compatible = compat(a_mapping[k].variable, b_mapping[k].variable)
             is_variable = True
         except AttributeError:
             # compare attribute value
-            if is_duck_array(a_mapping[k]) or is_duck_array(b_mapping[k]):
-                compatible = array_equiv(a_mapping[k], b_mapping[k])
-            else:
-                compatible = a_mapping[k] == b_mapping[k]
-
+            compatible = compare_attr(a_mapping[k], b_mapping[k])
             is_variable = False
 
         if not compatible:
@@ -782,19 +865,47 @@ def _diff_mapping_repr(
 
             if compat == "identical" and is_variable:
                 attrs_summary = []
+                a_attrs = a_mapping[k].attrs
+                b_attrs = b_mapping[k].attrs
 
+                attrs_to_print = set(a_attrs) ^ set(b_attrs)
+                attrs_to_print.update(
+                    {
+                        k
+                        for k in set(a_attrs) & set(b_attrs)
+                        if not compare_attr(a_attrs[k], b_attrs[k])
+                    }
+                )
                 for m in (a_mapping, b_mapping):
                     attr_s = "\n".join(
-                        summarize_attr(ak, av) for ak, av in m[k].attrs.items()
+                        "    " + summarize_attr(ak, av)
+                        for ak, av in m[k].attrs.items()
+                        if ak in attrs_to_print
                     )
+                    if attr_s:
+                        attr_s = "    Differing variable attributes:\n" + attr_s
                     attrs_summary.append(attr_s)
 
                 temp = [
                     "\n".join([var_s, attr_s]) if attr_s else var_s
-                    for var_s, attr_s in zip(temp, attrs_summary)
+                    for var_s, attr_s in zip(temp, attrs_summary, strict=True)
                 ]
 
-            diff_items += [ab_side + s[1:] for ab_side, s in zip(("L", "R"), temp)]
+                # TODO: It should be possible recursively use _diff_mapping_repr
+                #       instead of explicitly handling variable attrs specially.
+                #       That would require some refactoring.
+                # newdiff = _diff_mapping_repr(
+                #     {k: v for k,v in a_attrs.items() if k in attrs_to_print},
+                #     {k: v for k,v in b_attrs.items() if k in attrs_to_print},
+                #     compat=compat,
+                #     summarizer=summarize_attr,
+                #     title="Variable Attributes"
+                # )
+                # temp += [newdiff]
+
+            diff_items += [
+                ab_side + s[1:] for ab_side, s in zip(("L", "R"), temp, strict=True)
+            ]
 
     if diff_items:
         summary += [f"Differing {title.lower()}:"] + diff_items
@@ -815,8 +926,8 @@ def diff_coords_repr(a, b, compat, col_width=None):
         "Coordinates",
         summarize_variable,
         col_width=col_width,
-        a_indexes=a.indexes,
-        b_indexes=b.indexes,
+        a_indexes=a.xindexes,
+        b_indexes=b.xindexes,
     )
 
 
@@ -848,7 +959,9 @@ def diff_array_repr(a, b, compat):
         f"Left and right {type(a).__name__} objects are not {_compat_to_str(compat)}"
     ]
 
-    summary.append(diff_dim_summary(a, b))
+    if dims_diff := diff_dim_summary(a, b):
+        summary.append(dims_diff)
+
     if callable(compat):
         equiv = compat
     else:
@@ -858,22 +971,41 @@ def diff_array_repr(a, b, compat):
         temp = [wrap_indent(short_array_repr(obj), start="    ") for obj in (a, b)]
         diff_data_repr = [
             ab_side + "\n" + ab_data_repr
-            for ab_side, ab_data_repr in zip(("L", "R"), temp)
+            for ab_side, ab_data_repr in zip(("L", "R"), temp, strict=True)
         ]
         summary += ["Differing values:"] + diff_data_repr
 
     if hasattr(a, "coords"):
         col_width = _calculate_col_width(set(a.coords) | set(b.coords))
-        summary.append(
-            diff_coords_repr(a.coords, b.coords, compat, col_width=col_width)
-        )
+        if coords_diff := diff_coords_repr(
+            a.coords, b.coords, compat, col_width=col_width
+        ):
+            summary.append(coords_diff)
 
     if compat == "identical":
-        summary.append(diff_attrs_repr(a.attrs, b.attrs, compat))
+        if attrs_diff := diff_attrs_repr(a.attrs, b.attrs, compat):
+            summary.append(attrs_diff)
 
     return "\n".join(summary)
 
 
+def diff_treestructure(a: DataTree, b: DataTree) -> str | None:
+    """
+    Return a summary of why two trees are not isomorphic.
+    If they are isomorphic return None.
+    """
+    # .group_subtrees walks nodes in breadth-first-order, in order to produce as
+    # shallow of a diff as possible
+    for path, (node_a, node_b) in group_subtrees(a, b):
+        if node_a.children.keys() != node_b.children.keys():
+            path_str = "root node" if path == "." else f"node {path!r}"
+            child_summary = f"{list(node_a.children)} vs {list(node_b.children)}"
+            diff = f"Children at {path_str} do not match: {child_summary}"
+            return diff
+
+    return None
+
+
 def diff_dataset_repr(a, b, compat):
     summary = [
         f"Left and right {type(a).__name__} objects are not {_compat_to_str(compat)}"
@@ -881,13 +1013,211 @@ def diff_dataset_repr(a, b, compat):
 
     col_width = _calculate_col_width(set(list(a.variables) + list(b.variables)))
 
-    summary.append(diff_dim_summary(a, b))
-    summary.append(diff_coords_repr(a.coords, b.coords, compat, col_width=col_width))
-    summary.append(
-        diff_data_vars_repr(a.data_vars, b.data_vars, compat, col_width=col_width)
-    )
+    if dims_diff := diff_dim_summary(a, b):
+        summary.append(dims_diff)
+    if coords_diff := diff_coords_repr(a.coords, b.coords, compat, col_width=col_width):
+        summary.append(coords_diff)
+    if data_diff := diff_data_vars_repr(
+        a.data_vars, b.data_vars, compat, col_width=col_width
+    ):
+        summary.append(data_diff)
 
     if compat == "identical":
-        summary.append(diff_attrs_repr(a.attrs, b.attrs, compat))
+        if attrs_diff := diff_attrs_repr(a.attrs, b.attrs, compat):
+            summary.append(attrs_diff)
 
     return "\n".join(summary)
+
+
+def diff_nodewise_summary(a: DataTree, b: DataTree, compat):
+    """Iterates over all corresponding nodes, recording differences between data at each location."""
+
+    compat_str = _compat_to_str(compat)
+
+    summary = []
+    for path, (node_a, node_b) in group_subtrees(a, b):
+        a_ds, b_ds = node_a.dataset, node_b.dataset
+
+        if not a_ds._all_compat(b_ds, compat):
+            path_str = "root node" if path == "." else f"node {path!r}"
+            dataset_diff = diff_dataset_repr(a_ds, b_ds, compat_str)
+            data_diff = indent(
+                "\n".join(dataset_diff.split("\n", 1)[1:]), prefix="    "
+            )
+            nodediff = f"Data at {path_str} does not match:\n{data_diff}"
+            summary.append(nodediff)
+
+    return "\n\n".join(summary)
+
+
+def diff_datatree_repr(a: DataTree, b: DataTree, compat):
+    summary = [
+        f"Left and right {type(a).__name__} objects are not {_compat_to_str(compat)}"
+    ]
+
+    if compat == "identical":
+        if diff_name := diff_name_summary(a, b):
+            summary.append(diff_name)
+
+    treestructure_diff = diff_treestructure(a, b)
+
+    # If the trees structures are different there is no point comparing each node,
+    # and doing so would raise an error.
+    # TODO we could show any differences in nodes up to the first place that structure differs?
+    if treestructure_diff is not None:
+        summary.append(treestructure_diff)
+    elif compat != "isomorphic":
+        nodewise_diff = diff_nodewise_summary(a, b, compat)
+        summary.append(nodewise_diff)
+
+    return "\n\n".join(summary)
+
+
+def inherited_vars(mapping: ChainMap) -> dict:
+    return {k: v for k, v in mapping.parents.items() if k not in mapping.maps[0]}
+
+
+def _datatree_node_repr(node: DataTree, show_inherited: bool) -> str:
+    summary = [f"Group: {node.path}"]
+
+    col_width = _calculate_col_width(node.variables)
+    max_rows = OPTIONS["display_max_rows"]
+
+    inherited_coords = inherited_vars(node._coord_variables)
+
+    # Only show dimensions if also showing a variable or coordinates section.
+    show_dims = (
+        node._node_coord_variables
+        or (show_inherited and inherited_coords)
+        or node._data_variables
+    )
+
+    dim_sizes = node.sizes if show_inherited else node._node_dims
+
+    if show_dims:
+        # Includes inherited dimensions.
+        dims_start = pretty_print("Dimensions:", col_width)
+        dims_values = dim_summary_limited(
+            dim_sizes, col_width=col_width + 1, max_rows=max_rows
+        )
+        summary.append(f"{dims_start}({dims_values})")
+
+    if node._node_coord_variables:
+        node_coords = node.to_dataset(inherit=False).coords
+        summary.append(coords_repr(node_coords, col_width=col_width, max_rows=max_rows))
+
+    if show_inherited and inherited_coords:
+        summary.append(
+            inherited_coords_repr(node, col_width=col_width, max_rows=max_rows)
+        )
+
+    if show_dims:
+        unindexed_dims_str = unindexed_dims_repr(
+            dim_sizes, node.coords, max_rows=max_rows
+        )
+        if unindexed_dims_str:
+            summary.append(unindexed_dims_str)
+
+    if node._data_variables:
+        summary.append(
+            data_vars_repr(node._data_variables, col_width=col_width, max_rows=max_rows)
+        )
+
+    # TODO: only show indexes defined at this node, with a separate section for
+    # inherited indexes (if show_inherited=True)
+    display_default_indexes = _get_boolean_with_default(
+        "display_default_indexes", False
+    )
+    xindexes = filter_nondefault_indexes(
+        _get_indexes_dict(node.xindexes), not display_default_indexes
+    )
+    if xindexes:
+        summary.append(indexes_repr(xindexes, max_rows=max_rows))
+
+    if node.attrs:
+        summary.append(attrs_repr(node.attrs, max_rows=max_rows))
+
+    return "\n".join(summary)
+
+
+def datatree_repr(dt: DataTree) -> str:
+    """A printable representation of the structure of this entire tree."""
+    renderer = RenderDataTree(dt)
+
+    name_info = "" if dt.name is None else f" {dt.name!r}"
+    header = f"<xarray.DataTree{name_info}>"
+
+    lines = [header]
+    show_inherited = True
+    for pre, fill, node in renderer:
+        node_repr = _datatree_node_repr(node, show_inherited=show_inherited)
+        show_inherited = False  # only show inherited coords on the root
+
+        raw_repr_lines = node_repr.splitlines()
+
+        node_line = f"{pre}{raw_repr_lines[0]}"
+        lines.append(node_line)
+
+        for line in raw_repr_lines[1:]:
+            if len(node.children) > 0:
+                lines.append(f"{fill}{renderer.style.vertical}{line}")
+            else:
+                lines.append(f"{fill}{' ' * len(renderer.style.vertical)}{line}")
+
+    return "\n".join(lines)
+
+
+def shorten_list_repr(items: Sequence, max_items: int) -> str:
+    if len(items) <= max_items:
+        return repr(items)
+    else:
+        first_half = repr(items[: max_items // 2])[
+            1:-1
+        ]  # Convert to string and remove brackets
+        second_half = repr(items[-max_items // 2 :])[
+            1:-1
+        ]  # Convert to string and remove brackets
+        return f"[{first_half}, ..., {second_half}]"
+
+
+def render_human_readable_nbytes(
+    nbytes: int,
+    /,
+    *,
+    attempt_constant_width: bool = False,
+) -> str:
+    """Renders simple human-readable byte count representation
+
+    This is only a quick representation that should not be relied upon for precise needs.
+
+    To get the exact byte count, please use the ``nbytes`` attribute directly.
+
+    Parameters
+    ----------
+    nbytes
+        Byte count
+    attempt_constant_width
+        For reasonable nbytes sizes, tries to render a fixed-width representation.
+
+    Returns
+    -------
+        Human-readable representation of the byte count
+    """
+    dividend = float(nbytes)
+    divisor = 1000.0
+    last_unit_available = UNITS[-1]
+
+    for unit in UNITS:
+        if dividend < divisor or unit == last_unit_available:
+            break
+        dividend /= divisor
+
+    dividend_str = f"{dividend:.0f}"
+    unit_str = f"{unit}"
+
+    if attempt_constant_width:
+        dividend_str = dividend_str.rjust(3)
+        unit_str = unit_str.ljust(2)
+
+    string = f"{dividend_str}{unit_str}"
+    return string
