@@ -4,17 +4,21 @@ import functools
 import itertools
 import math
 import warnings
-from collections.abc import Hashable, Iterator, Mapping
-from typing import TYPE_CHECKING, Any, Callable, Generic, TypeVar
+from collections.abc import Callable, Hashable, Iterator, Mapping
+from typing import TYPE_CHECKING, Any, Generic, TypeVar
 
 import numpy as np
 
-from xarray.core import dtypes, duck_array_ops, utils
+from xarray.core import dask_array_ops, dtypes, duck_array_ops, utils
 from xarray.core.arithmetic import CoarsenArithmetic
 from xarray.core.options import OPTIONS, _get_keep_attrs
-from xarray.core.pycompat import is_duck_dask_array
 from xarray.core.types import CoarsenBoundaryOptions, SideOptions, T_Xarray
-from xarray.core.utils import either_dict_or_kwargs
+from xarray.core.utils import (
+    either_dict_or_kwargs,
+    is_duck_dask_array,
+    module_available,
+)
+from xarray.util.deprecation_helpers import _deprecate_positional_args
 
 try:
     import bottleneck
@@ -59,8 +63,13 @@ class Rolling(Generic[T_Xarray]):
     xarray.DataArray.rolling
     """
 
-    __slots__ = ("obj", "window", "min_periods", "center", "dim")
+    __slots__ = ("center", "dim", "min_periods", "obj", "window")
     _attributes = ("window", "min_periods", "center", "dim")
+    dim: list[Hashable]
+    window: list[int]
+    center: list[bool]
+    obj: T_Xarray
+    min_periods: int
 
     def __init__(
         self,
@@ -91,8 +100,8 @@ class Rolling(Generic[T_Xarray]):
         -------
         rolling : type of input argument
         """
-        self.dim: list[Hashable] = []
-        self.window: list[int] = []
+        self.dim = []
+        self.window = []
         for d, w in windows.items():
             self.dim.append(d)
             if w <= 0:
@@ -100,7 +109,15 @@ class Rolling(Generic[T_Xarray]):
             self.window.append(w)
 
         self.center = self._mapping_to_list(center, default=False)
-        self.obj: T_Xarray = obj
+        self.obj = obj
+
+        missing_dims = tuple(dim for dim in self.dim if dim not in self.obj.dims)
+        if missing_dims:
+            # NOTE: we raise KeyError here but ValueError in Coarsen.
+            raise KeyError(
+                f"Window dimensions {missing_dims} not found in {self.obj.__class__.__name__} "
+                f"dimensions {tuple(self.obj.dims)}"
+            )
 
         # attributes
         if min_periods is not None and min_periods <= 0:
@@ -115,7 +132,7 @@ class Rolling(Generic[T_Xarray]):
 
         attrs = [
             "{k}->{v}{c}".format(k=k, v=w, c="(center)" if c else "")
-            for k, w, c in zip(self.dim, self.window, self.center)
+            for k, w, c in zip(self.dim, self.window, self.center, strict=True)
         ]
         return "{klass} [{attrs}]".format(
             klass=self.__class__.__name__, attrs=",".join(attrs)
@@ -129,10 +146,21 @@ class Rolling(Generic[T_Xarray]):
         return len(self.dim)
 
     def _reduce_method(  # type: ignore[misc]
-        name: str, fillna: Any, rolling_agg_func: Callable | None = None
+        name: str,
+        fillna: Any,
+        rolling_agg_func: Callable | None = None,
+        automatic_rechunk: bool = False,
     ) -> Callable[..., T_Xarray]:
         """Constructs reduction methods built on a numpy reduction function (e.g. sum),
-        a bottleneck reduction function (e.g. move_sum), or a Rolling reduction (_mean).
+        a numbagg reduction function (e.g. move_sum), a bottleneck reduction function
+        (e.g. move_sum), or a Rolling reduction (_mean).
+
+        The logic here for which function to run is quite diffuse, across this method &
+        _array_reduce. Arguably we could refactor this. But one constraint is that we
+        need context of xarray options, of the functions each library offers, of
+        the array (e.g. dtype).
+
+        Set automatic_rechunk=True when the reduction method makes a memory copy.
         """
         if rolling_agg_func:
             array_agg_func = None
@@ -140,16 +168,24 @@ class Rolling(Generic[T_Xarray]):
             array_agg_func = getattr(duck_array_ops, name)
 
         bottleneck_move_func = getattr(bottleneck, "move_" + name, None)
+        if module_available("numbagg"):
+            import numbagg
+
+            numbagg_move_func = getattr(numbagg, "move_" + name, None)
+        else:
+            numbagg_move_func = None
 
         def method(self, keep_attrs=None, **kwargs):
             keep_attrs = self._get_keep_attrs(keep_attrs)
 
-            return self._numpy_or_bottleneck_reduce(
-                array_agg_func,
-                bottleneck_move_func,
-                rolling_agg_func,
+            return self._array_reduce(
+                array_agg_func=array_agg_func,
+                bottleneck_move_func=bottleneck_move_func,
+                numbagg_move_func=numbagg_move_func,
+                rolling_agg_func=rolling_agg_func,
                 keep_attrs=keep_attrs,
                 fillna=fillna,
+                sliding_window_view_kwargs=dict(automatic_rechunk=automatic_rechunk),
                 **kwargs,
             )
 
@@ -167,16 +203,19 @@ class Rolling(Generic[T_Xarray]):
 
     _mean.__doc__ = _ROLLING_REDUCE_DOCSTRING_TEMPLATE.format(name="mean")
 
-    argmax = _reduce_method("argmax", dtypes.NINF)
-    argmin = _reduce_method("argmin", dtypes.INF)
+    # automatic_rechunk is set to True for reductions that make a copy.
+    # std, var could be optimized after which we can set it to False
+    # See #4325
+    argmax = _reduce_method("argmax", dtypes.NINF, automatic_rechunk=True)
+    argmin = _reduce_method("argmin", dtypes.INF, automatic_rechunk=True)
     max = _reduce_method("max", dtypes.NINF)
     min = _reduce_method("min", dtypes.INF)
     prod = _reduce_method("prod", 1)
     sum = _reduce_method("sum", 0)
     mean = _reduce_method("mean", None, _mean)
-    std = _reduce_method("std", None)
-    var = _reduce_method("var", None)
-    median = _reduce_method("median", None)
+    std = _reduce_method("std", None, automatic_rechunk=True)
+    var = _reduce_method("var", None, automatic_rechunk=True)
+    median = _reduce_method("median", None, automatic_rechunk=True)
 
     def _counts(self, keep_attrs: bool | None) -> T_Xarray:
         raise NotImplementedError()
@@ -243,7 +282,8 @@ class DataArrayRolling(Rolling["DataArray"]):
             (otherwise result is NA). The default, None, is equivalent to
             setting min_periods equal to the size of the window.
         center : bool, default: False
-            Set the labels at the center of the window.
+            Set the labels at the center of the window. The default, False,
+            sets the labels at the right edge of the window.
 
         Returns
         -------
@@ -272,7 +312,7 @@ class DataArrayRolling(Rolling["DataArray"]):
         starts = stops - window0
         starts[: window0 - offset] = 0
 
-        for label, start, stop in zip(self.window_labels, starts, stops):
+        for label, start, stop in zip(self.window_labels, starts, stops, strict=True):
             window = self.obj.isel({dim0: slice(start, stop)})
 
             counts = window.count(dim=[dim0])
@@ -280,12 +320,15 @@ class DataArrayRolling(Rolling["DataArray"]):
 
             yield (label, window)
 
+    @_deprecate_positional_args("v2024.11.0")
     def construct(
         self,
         window_dim: Hashable | Mapping[Any, Hashable] | None = None,
+        *,
         stride: int | Mapping[Any, int] = 1,
         fill_value: Any = dtypes.NA,
         keep_attrs: bool | None = None,
+        sliding_window_view_kwargs: Mapping[Any, Any] | None = None,
         **window_dim_kwargs: Hashable,
     ) -> DataArray:
         """
@@ -304,13 +347,31 @@ class DataArrayRolling(Rolling["DataArray"]):
             If True, the attributes (``attrs``) will be copied from the original
             object to the new one. If False, the new object will be returned
             without attributes. If None uses the global default.
+        sliding_window_view_kwargs : Mapping
+            Keyword arguments that should be passed to the underlying array type's
+            ``sliding_window_view`` function.
         **window_dim_kwargs : Hashable, optional
             The keyword arguments form of ``window_dim`` {dim: new_name, ...}.
 
         Returns
         -------
-        DataArray that is a view of the original array. The returned array is
-        not writeable.
+        DataArray
+            a view of the original array. By default, the returned array is not writeable.
+            For numpy arrays, one can pass ``writeable=True`` in ``sliding_window_view_kwargs``.
+
+        See Also
+        --------
+        numpy.lib.stride_tricks.sliding_window_view
+        dask.array.lib.stride_tricks.sliding_window_view
+
+        Notes
+        -----
+        With dask arrays, it's possible to pass the ``automatic_rechunk`` kwarg as
+        ``sliding_window_view_kwargs={"automatic_rechunk": True}``. This controls
+        whether dask should automatically rechunk the output to avoid
+        exploding chunk sizes. Automatically rechunking is the default behaviour.
+        Importantly, each chunk will be a view of the data so large chunk sizes are
+        only safe if *no* copies are made later.
 
         Examples
         --------
@@ -318,7 +379,7 @@ class DataArrayRolling(Rolling["DataArray"]):
 
         >>> rolling = da.rolling(b=3)
         >>> rolling.construct("window_dim")
-        <xarray.DataArray (a: 2, b: 4, window_dim: 3)>
+        <xarray.DataArray (a: 2, b: 4, window_dim: 3)> Size: 192B
         array([[[nan, nan,  0.],
                 [nan,  0.,  1.],
                 [ 0.,  1.,  2.],
@@ -332,7 +393,7 @@ class DataArrayRolling(Rolling["DataArray"]):
 
         >>> rolling = da.rolling(b=3, center=True)
         >>> rolling.construct("window_dim")
-        <xarray.DataArray (a: 2, b: 4, window_dim: 3)>
+        <xarray.DataArray (a: 2, b: 4, window_dim: 3)> Size: 192B
         array([[[nan,  0.,  1.],
                 [ 0.,  1.,  2.],
                 [ 1.,  2.,  3.],
@@ -346,26 +407,34 @@ class DataArrayRolling(Rolling["DataArray"]):
 
         """
 
+        if sliding_window_view_kwargs is None:
+            sliding_window_view_kwargs = {}
         return self._construct(
             self.obj,
             window_dim=window_dim,
             stride=stride,
             fill_value=fill_value,
             keep_attrs=keep_attrs,
+            sliding_window_view_kwargs=sliding_window_view_kwargs,
             **window_dim_kwargs,
         )
 
     def _construct(
         self,
         obj: DataArray,
+        *,
         window_dim: Hashable | Mapping[Any, Hashable] | None = None,
         stride: int | Mapping[Any, int] = 1,
         fill_value: Any = dtypes.NA,
         keep_attrs: bool | None = None,
+        sliding_window_view_kwargs: Mapping[Any, Any] | None = None,
         **window_dim_kwargs: Hashable,
     ) -> DataArray:
         from xarray.core.dataarray import DataArray
 
+        if sliding_window_view_kwargs is None:
+            sliding_window_view_kwargs = {}
+
         keep_attrs = self._get_keep_attrs(keep_attrs)
 
         if window_dim is None:
@@ -381,7 +450,12 @@ class DataArrayRolling(Rolling["DataArray"]):
         strides = self._mapping_to_list(stride, default=1)
 
         window = obj.variable.rolling_window(
-            self.dim, self.window, window_dims, self.center, fill_value=fill_value
+            self.dim,
+            self.window,
+            window_dims,
+            center=self.center,
+            fill_value=fill_value,
+            **sliding_window_view_kwargs,
         )
 
         attrs = obj.attrs if keep_attrs else {}
@@ -393,13 +467,21 @@ class DataArrayRolling(Rolling["DataArray"]):
             attrs=attrs,
             name=obj.name,
         )
-        return result.isel({d: slice(None, None, s) for d, s in zip(self.dim, strides)})
+        return result.isel(
+            {d: slice(None, None, s) for d, s in zip(self.dim, strides, strict=True)}
+        )
 
     def reduce(
-        self, func: Callable, keep_attrs: bool | None = None, **kwargs: Any
+        self,
+        func: Callable,
+        keep_attrs: bool | None = None,
+        *,
+        sliding_window_view_kwargs: Mapping[Any, Any] | None = None,
+        **kwargs: Any,
     ) -> DataArray:
-        """Reduce the items in this group by applying `func` along some
-        dimension(s).
+        """Reduce each window by applying `func`.
+
+        Equivalent to ``.construct(...).reduce(func, ...)``.
 
         Parameters
         ----------
@@ -411,6 +493,9 @@ class DataArrayRolling(Rolling["DataArray"]):
             If True, the attributes (``attrs``) will be copied from the original
             object to the new one. If False, the new object will be returned
             without attributes. If None uses the global default.
+        sliding_window_view_kwargs
+            Keyword arguments that should be passed to the underlying array type's
+            ``sliding_window_view`` function.
         **kwargs : dict
             Additional keyword arguments passed on to `func`.
 
@@ -419,12 +504,26 @@ class DataArrayRolling(Rolling["DataArray"]):
         reduced : DataArray
             Array with summarized data.
 
+        See Also
+        --------
+        numpy.lib.stride_tricks.sliding_window_view
+        dask.array.lib.stride_tricks.sliding_window_view
+
+        Notes
+        -----
+        With dask arrays, it's possible to pass the ``automatic_rechunk`` kwarg as
+        ``sliding_window_view_kwargs={"automatic_rechunk": True}``. This controls
+        whether dask should automatically rechunk the output to avoid
+        exploding chunk sizes. Automatically rechunking is the default behaviour.
+        Importantly, each chunk will be a view of the data so large chunk sizes are
+        only safe if *no* copies are made later.
+
         Examples
         --------
         >>> da = xr.DataArray(np.arange(8).reshape(2, 4), dims=("a", "b"))
         >>> rolling = da.rolling(b=3)
         >>> rolling.construct("window_dim")
-        <xarray.DataArray (a: 2, b: 4, window_dim: 3)>
+        <xarray.DataArray (a: 2, b: 4, window_dim: 3)> Size: 192B
         array([[[nan, nan,  0.],
                 [nan,  0.,  1.],
                 [ 0.,  1.,  2.],
@@ -437,14 +536,14 @@ class DataArrayRolling(Rolling["DataArray"]):
         Dimensions without coordinates: a, b, window_dim
 
         >>> rolling.reduce(np.sum)
-        <xarray.DataArray (a: 2, b: 4)>
+        <xarray.DataArray (a: 2, b: 4)> Size: 64B
         array([[nan, nan,  3.,  6.],
                [nan, nan, 15., 18.]])
         Dimensions without coordinates: a, b
 
         >>> rolling = da.rolling(b=3, min_periods=1)
         >>> rolling.reduce(np.nansum)
-        <xarray.DataArray (a: 2, b: 4)>
+        <xarray.DataArray (a: 2, b: 4)> Size: 64B
         array([[ 0.,  1.,  3.,  6.],
                [ 4.,  9., 15., 18.]])
         Dimensions without coordinates: a, b
@@ -464,12 +563,15 @@ class DataArrayRolling(Rolling["DataArray"]):
         else:
             obj = self.obj
         windows = self._construct(
-            obj, rolling_dim, keep_attrs=keep_attrs, fill_value=fillna
+            obj,
+            window_dim=rolling_dim,
+            keep_attrs=keep_attrs,
+            fill_value=fillna,
+            sliding_window_view_kwargs=sliding_window_view_kwargs,
         )
 
-        result = windows.reduce(
-            func, dim=list(rolling_dim.values()), keep_attrs=keep_attrs, **kwargs
-        )
+        dim = list(rolling_dim.values())
+        result = windows.reduce(func, dim=dim, keep_attrs=keep_attrs, **kwargs)
 
         # Find valid windows based on count.
         counts = self._counts(keep_attrs=False)
@@ -486,20 +588,59 @@ class DataArrayRolling(Rolling["DataArray"]):
         # array is faster to be reduced than object array.
         # The use of skipna==False is also faster since it does not need to
         # copy the strided array.
+        dim = list(rolling_dim.values())
         counts = (
             self.obj.notnull(keep_attrs=keep_attrs)
             .rolling(
-                {d: w for d, w in zip(self.dim, self.window)},
+                dict(zip(self.dim, self.window, strict=True)),
                 center={d: self.center[i] for i, d in enumerate(self.dim)},
             )
             .construct(rolling_dim, fill_value=False, keep_attrs=keep_attrs)
-            .sum(dim=list(rolling_dim.values()), skipna=False, keep_attrs=keep_attrs)
+            .sum(dim=dim, skipna=False, keep_attrs=keep_attrs)
         )
         return counts
 
-    def _bottleneck_reduce(self, func, keep_attrs, **kwargs):
-        from xarray.core.dataarray import DataArray
+    def _numbagg_reduce(self, func, keep_attrs, **kwargs):
+        # Some of this is copied from `_bottleneck_reduce`, we could reduce this as part
+        # of a wider refactor.
 
+        axis = self.obj.get_axis_num(self.dim[0])
+
+        padded = self.obj.variable
+        if self.center[0]:
+            if is_duck_dask_array(padded.data):
+                # workaround to make the padded chunk size larger than
+                # self.window - 1
+                shift = -(self.window[0] + 1) // 2
+                offset = (self.window[0] - 1) // 2
+                valid = (slice(None),) * axis + (
+                    slice(offset, offset + self.obj.shape[axis]),
+                )
+            else:
+                shift = (-self.window[0] // 2) + 1
+                valid = (slice(None),) * axis + (slice(-shift, None),)
+            padded = padded.pad({self.dim[0]: (0, -shift)}, mode="constant")
+
+        if is_duck_dask_array(padded.data) and False:
+            raise AssertionError("should not be reachable")
+        else:
+            values = func(
+                padded.data,
+                window=self.window[0],
+                min_count=self.min_periods,
+                axis=axis,
+            )
+
+        if self.center[0]:
+            values = values[valid]
+
+        attrs = self.obj.attrs if keep_attrs else {}
+
+        return self.obj.__class__(
+            values, self.obj.coords, attrs=attrs, name=self.obj.name
+        )
+
+    def _bottleneck_reduce(self, func, keep_attrs, **kwargs):
         # bottleneck doesn't allow min_count to be 0, although it should
         # work the same as if min_count = 1
         # Note bottleneck only works with 1d-rolling.
@@ -526,23 +667,33 @@ class DataArrayRolling(Rolling["DataArray"]):
             padded = padded.pad({self.dim[0]: (0, -shift)}, mode="constant")
 
         if is_duck_dask_array(padded.data):
-            raise AssertionError("should not be reachable")
+            values = dask_array_ops.dask_rolling_wrapper(
+                func, padded, axis=axis, window=self.window[0], min_count=min_count
+            )
         else:
             values = func(
                 padded.data, window=self.window[0], min_count=min_count, axis=axis
             )
+        # index 0 is at the rightmost edge of the window
+        # need to reverse index here
+        # see GH #8541
+        if func in [bottleneck.move_argmin, bottleneck.move_argmax]:
+            values = self.window[0] - 1 - values
 
         if self.center[0]:
             values = values[valid]
 
         attrs = self.obj.attrs if keep_attrs else {}
 
-        return DataArray(values, self.obj.coords, attrs=attrs, name=self.obj.name)
+        return self.obj.__class__(
+            values, self.obj.coords, attrs=attrs, name=self.obj.name
+        )
 
-    def _numpy_or_bottleneck_reduce(
+    def _array_reduce(
         self,
         array_agg_func,
         bottleneck_move_func,
+        numbagg_move_func,
         rolling_agg_func,
         keep_attrs,
         fillna,
@@ -558,20 +709,53 @@ class DataArrayRolling(Rolling["DataArray"]):
             )
             del kwargs["dim"]
 
+        xp = duck_array_ops.get_array_namespace(self.obj.data)
+        if (
+            OPTIONS["use_numbagg"]
+            and module_available("numbagg")
+            and numbagg_move_func is not None
+            # TODO: we could at least allow this for the equivalent of `apply_ufunc`'s
+            # "parallelized". `rolling_exp` does this, as an example (but rolling_exp is
+            # much simpler)
+            and not is_duck_dask_array(self.obj.data)
+            # Numbagg doesn't handle object arrays and generally has dtype consistency,
+            # so doesn't deal well with bool arrays which are expected to change type.
+            and self.obj.data.dtype.kind not in "ObMm"
+            # TODO: we could also allow this, probably as part of a refactoring of this
+            # module, so we can use the machinery in `self.reduce`.
+            and self.ndim == 1
+            and xp is np
+        ):
+            import numbagg
+
+            # Numbagg has a default ddof of 1. I (@max-sixty) think we should make
+            # this the default in xarray too, but until we do, don't use numbagg for
+            # std and var unless ddof is set to 1.
+            if (
+                numbagg_move_func not in [numbagg.move_std, numbagg.move_var]
+                or kwargs.get("ddof") == 1
+            ):
+                return self._numbagg_reduce(
+                    numbagg_move_func, keep_attrs=keep_attrs, **kwargs
+                )
+
         if (
             OPTIONS["use_bottleneck"]
             and bottleneck_move_func is not None
-            and not is_duck_dask_array(self.obj.data)
+            and (
+                not is_duck_dask_array(self.obj.data)
+                or module_available("dask", "2024.11.0")
+            )
             and self.ndim == 1
+            and xp is np
         ):
-            # TODO: renable bottleneck with dask after the issues
-            # underlying https://github.com/pydata/xarray/issues/2940 are
-            # fixed.
             return self._bottleneck_reduce(
                 bottleneck_move_func, keep_attrs=keep_attrs, **kwargs
             )
+
         if rolling_agg_func:
             return rolling_agg_func(self, keep_attrs=self._get_keep_attrs(keep_attrs))
+
         if fillna is not None:
             if fillna is dtypes.INF:
                 fillna = dtypes.get_pos_infinity(self.obj.dtype, max_for_int=True)
@@ -610,7 +794,8 @@ class DatasetRolling(Rolling["Dataset"]):
             (otherwise result is NA). The default, None, is equivalent to
             setting min_periods equal to the size of the window.
         center : bool or mapping of hashable to bool, default: False
-            Set the labels at the center of the window.
+            Set the labels at the center of the window. The default, False,
+            sets the labels at the right edge of the window.
 
         Returns
         -------
@@ -624,8 +809,7 @@ class DatasetRolling(Rolling["Dataset"]):
         xarray.DataArray.groupby
         """
         super().__init__(obj, windows, min_periods, center)
-        if any(d not in self.obj.dims for d in self.dim):
-            raise KeyError(self.dim)
+
         # Keep each Rolling object as a dictionary
         self.rollings = {}
         for key, da in self.obj.data_vars.items():
@@ -659,7 +843,11 @@ class DatasetRolling(Rolling["Dataset"]):
         return Dataset(reduced, coords=self.obj.coords, attrs=attrs)
 
     def reduce(
-        self, func: Callable, keep_attrs: bool | None = None, **kwargs: Any
+        self,
+        func: Callable,
+        keep_attrs: bool | None = None,
+        sliding_window_view_kwargs: Mapping[Any, Any] | None = None,
+        **kwargs: Any,
     ) -> DataArray:
         """Reduce the items in this group by applying `func` along some
         dimension(s).
@@ -674,6 +862,9 @@ class DatasetRolling(Rolling["Dataset"]):
             If True, the attributes (``attrs``) will be copied from the original
             object to the new one. If False, the new object will be returned
             without attributes. If None uses the global default.
+        sliding_window_view_kwargs : Mapping
+            Keyword arguments that should be passed to the underlying array type's
+            ``sliding_window_view`` function.
         **kwargs : dict
             Additional keyword arguments passed on to `func`.
 
@@ -681,10 +872,25 @@ class DatasetRolling(Rolling["Dataset"]):
         -------
         reduced : DataArray
             Array with summarized data.
+
+        See Also
+        --------
+        numpy.lib.stride_tricks.sliding_window_view
+        dask.array.lib.stride_tricks.sliding_window_view
+
+        Notes
+        -----
+        With dask arrays, it's possible to pass the ``automatic_rechunk`` kwarg as
+        ``sliding_window_view_kwargs={"automatic_rechunk": True}``. This controls
+        whether dask should automatically rechunk the output to avoid
+        exploding chunk sizes. Automatically rechunking is the default behaviour.
+        Importantly, each chunk will be a view of the data so large chunk sizes are
+        only safe if *no* copies are made later.
         """
         return self._dataset_implementation(
             functools.partial(DataArrayRolling.reduce, func=func),
             keep_attrs=keep_attrs,
+            sliding_window_view_kwargs=sliding_window_view_kwargs,
             **kwargs,
         )
 
@@ -693,7 +899,7 @@ class DatasetRolling(Rolling["Dataset"]):
             DataArrayRolling._counts, keep_attrs=keep_attrs
         )
 
-    def _numpy_or_bottleneck_reduce(
+    def _array_reduce(
         self,
         array_agg_func,
         bottleneck_move_func,
@@ -703,7 +909,7 @@ class DatasetRolling(Rolling["Dataset"]):
     ):
         return self._dataset_implementation(
             functools.partial(
-                DataArrayRolling._numpy_or_bottleneck_reduce,
+                DataArrayRolling._array_reduce,
                 array_agg_func=array_agg_func,
                 bottleneck_move_func=bottleneck_move_func,
                 rolling_agg_func=rolling_agg_func,
@@ -712,12 +918,15 @@ class DatasetRolling(Rolling["Dataset"]):
             **kwargs,
         )
 
+    @_deprecate_positional_args("v2024.11.0")
     def construct(
         self,
         window_dim: Hashable | Mapping[Any, Hashable] | None = None,
+        *,
         stride: int | Mapping[Any, int] = 1,
         fill_value: Any = dtypes.NA,
         keep_attrs: bool | None = None,
+        sliding_window_view_kwargs: Mapping[Any, Any] | None = None,
         **window_dim_kwargs: Hashable,
     ) -> Dataset:
         """
@@ -733,12 +942,31 @@ class DatasetRolling(Rolling["Dataset"]):
             size of stride for the rolling window.
         fill_value : Any, default: dtypes.NA
             Filling value to match the dimension size.
+        sliding_window_view_kwargs
+            Keyword arguments that should be passed to the underlying array type's
+            ``sliding_window_view`` function.
         **window_dim_kwargs : {dim: new_name, ...}, optional
             The keyword arguments form of ``window_dim``.
 
         Returns
         -------
-        Dataset with variables converted from rolling object.
+        Dataset
+            Dataset with views of the original arrays. By default, the returned arrays are not writeable.
+            For numpy arrays, one can pass ``writeable=True`` in ``sliding_window_view_kwargs``.
+
+        See Also
+        --------
+        numpy.lib.stride_tricks.sliding_window_view
+        dask.array.lib.stride_tricks.sliding_window_view
+
+        Notes
+        -----
+        With dask arrays, it's possible to pass the ``automatic_rechunk`` kwarg as
+        ``sliding_window_view_kwargs={"automatic_rechunk": True}``. This controls
+        whether dask should automatically rechunk the output to avoid
+        exploding chunk sizes. Automatically rechunking is the default behaviour.
+        Importantly, each chunk will be a view of the data so large chunk sizes are
+        only safe if *no* copies are made later.
         """
 
         from xarray.core.dataset import Dataset
@@ -770,6 +998,7 @@ class DatasetRolling(Rolling["Dataset"]):
                     fill_value=fill_value,
                     stride=st,
                     keep_attrs=keep_attrs,
+                    sliding_window_view_kwargs=sliding_window_view_kwargs,
                 )
             else:
                 dataset[key] = da.copy()
@@ -778,11 +1007,14 @@ class DatasetRolling(Rolling["Dataset"]):
             if not keep_attrs:
                 dataset[key].attrs = {}
 
+        # Need to stride coords as well. TODO: is there a better way?
+        coords = self.obj.isel(
+            {d: slice(None, None, s) for d, s in zip(self.dim, strides, strict=True)}
+        ).coords
+
         attrs = self.obj.attrs if keep_attrs else {}
 
-        return Dataset(dataset, coords=self.obj.coords, attrs=attrs).isel(
-            {d: slice(None, None, s) for d, s in zip(self.dim, strides)}
-        )
+        return Dataset(dataset, coords=coords, attrs=attrs)
 
 
 class Coarsen(CoarsenArithmetic, Generic[T_Xarray]):
@@ -795,15 +1027,19 @@ class Coarsen(CoarsenArithmetic, Generic[T_Xarray]):
     """
 
     __slots__ = (
-        "obj",
         "boundary",
         "coord_func",
-        "windows",
+        "obj",
         "side",
         "trim_excess",
+        "windows",
     )
     _attributes = ("windows", "side", "trim_excess")
     obj: T_Xarray
+    windows: Mapping[Hashable, int]
+    side: SideOptions | Mapping[Hashable, SideOptions]
+    boundary: CoarsenBoundaryOptions
+    coord_func: Mapping[Hashable, str | Callable]
 
     def __init__(
         self,
@@ -833,23 +1069,28 @@ class Coarsen(CoarsenArithmetic, Generic[T_Xarray]):
         Returns
         -------
         coarsen
+
         """
         self.obj = obj
         self.windows = windows
         self.side = side
         self.boundary = boundary
 
-        absent_dims = [dim for dim in windows.keys() if dim not in self.obj.dims]
-        if absent_dims:
+        missing_dims = tuple(dim for dim in windows.keys() if dim not in self.obj.dims)
+        if missing_dims:
             raise ValueError(
-                f"Dimensions {absent_dims!r} not found in {self.obj.__class__.__name__}."
+                f"Window dimensions {missing_dims} not found in {self.obj.__class__.__name__} "
+                f"dimensions {tuple(self.obj.dims)}"
             )
-        if not utils.is_dict_like(coord_func):
-            coord_func = {d: coord_func for d in self.obj.dims}  # type: ignore[misc]
+
+        if utils.is_dict_like(coord_func):
+            coord_func_map = coord_func
+        else:
+            coord_func_map = {d: coord_func for d in self.obj.dims}
         for c in self.obj.coords:
-            if c not in coord_func:
-                coord_func[c] = duck_array_ops.mean  # type: ignore[index]
-        self.coord_func: Mapping[Hashable, str | Callable] = coord_func
+            if c not in coord_func_map:
+                coord_func_map[c] = duck_array_ops.mean  # type: ignore[index]
+        self.coord_func = coord_func_map
 
     def _get_keep_attrs(self, keep_attrs):
         if keep_attrs is None:
@@ -899,7 +1140,7 @@ class Coarsen(CoarsenArithmetic, Generic[T_Xarray]):
         --------
         >>> da = xr.DataArray(np.arange(24), dims="time")
         >>> da.coarsen(time=12).construct(time=("year", "month"))
-        <xarray.DataArray (year: 2, month: 12)>
+        <xarray.DataArray (year: 2, month: 12)> Size: 192B
         array([[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11],
                [12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]])
         Dimensions without coordinates: year, month
@@ -1055,7 +1296,7 @@ class DataArrayCoarsen(Coarsen["DataArray"]):
         >>> da = xr.DataArray(np.arange(8).reshape(2, 4), dims=("a", "b"))
         >>> coarsen = da.coarsen(b=2)
         >>> coarsen.reduce(np.sum)
-        <xarray.DataArray (a: 2, b: 2)>
+        <xarray.DataArray (a: 2, b: 2)> Size: 32B
         array([[ 1,  5],
                [ 9, 13]])
         Dimensions without coordinates: a, b
