@@ -9,15 +9,19 @@ from __future__ import annotations
 import datetime
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
+from itertools import pairwise
 from typing import TYPE_CHECKING, Any, Literal, cast
 
 import numpy as np
 import pandas as pd
+from numpy.typing import ArrayLike
 
 from xarray.coding.cftime_offsets import BaseCFTimeOffset, _new_to_legacy_freq
 from xarray.core import duck_array_ops
+from xarray.core.computation import apply_ufunc
 from xarray.core.coordinates import Coordinates, _coordinates_from_variable
 from xarray.core.dataarray import DataArray
+from xarray.core.duck_array_ops import isnull
 from xarray.core.groupby import T_Group, _DummyGroup
 from xarray.core.indexes import safe_cast_to_index
 from xarray.core.resample_cftime import CFTimeGrouper
@@ -26,17 +30,19 @@ from xarray.core.types import (
     DatetimeLike,
     GroupIndices,
     ResampleCompatible,
+    Self,
     SideOptions,
 )
 from xarray.core.variable import Variable
+from xarray.namedarray.pycompat import is_chunked_array
 
 __all__ = [
+    "BinGrouper",
     "EncodedGroups",
     "Grouper",
     "Resampler",
-    "UniqueGrouper",
-    "BinGrouper",
     "TimeResampler",
+    "UniqueGrouper",
 ]
 
 RESAMPLE_DIM = "__resample_dim__"
@@ -86,11 +92,17 @@ class EncodedGroups:
         self.full_index = full_index
 
         if group_indices is None:
-            self.group_indices = tuple(
-                g
-                for g in _codes_to_group_indices(codes.data.ravel(), len(full_index))
-                if g
-            )
+            if not is_chunked_array(codes.data):
+                self.group_indices = tuple(
+                    g
+                    for g in _codes_to_group_indices(
+                        codes.data.ravel(), len(full_index)
+                    )
+                    if g
+                )
+            else:
+                # We will not use this when grouping by a chunked array
+                self.group_indices = tuple()
         else:
             self.group_indices = group_indices
 
@@ -128,6 +140,13 @@ class Grouper(ABC):
         """
         pass
 
+    @abstractmethod
+    def reset(self) -> Self:
+        """
+        Creates a new version of this Grouper clearing any caches.
+        """
+        pass
+
 
 class Resampler(Grouper):
     """
@@ -141,9 +160,20 @@ class Resampler(Grouper):
 
 @dataclass
 class UniqueGrouper(Grouper):
-    """Grouper object for grouping by a categorical variable."""
+    """
+    Grouper object for grouping by a categorical variable.
+
+    Parameters
+    ----------
+    labels: array-like, optional
+        Group labels to aggregate on. This is required when grouping by a chunked array type
+        (e.g. dask or cubed) since it is used to construct the coordinate on the output.
+        Grouped operations will only be run on the specified group labels. Any group that is not
+        present in ``labels`` will be ignored.
+    """
 
     _group_as_index: pd.Index | None = field(default=None, repr=False)
+    labels: ArrayLike | None = field(default=None)
 
     @property
     def group_as_index(self) -> pd.Index:
@@ -155,9 +185,20 @@ class UniqueGrouper(Grouper):
                 self._group_as_index = pd.Index(np.array(self.group).ravel())
         return self._group_as_index
 
+    def reset(self) -> Self:
+        return type(self)()
+
     def factorize(self, group: T_Group) -> EncodedGroups:
         self.group = group
 
+        if is_chunked_array(group.data) and self.labels is None:
+            raise ValueError(
+                "When grouping by a dask array, `labels` must be passed using "
+                "a UniqueGrouper object."
+            )
+        if self.labels is not None:
+            return self._factorize_given_labels(group)
+
         index = self.group_as_index
         is_unique_and_monotonic = isinstance(self.group, _DummyGroup) or (
             index.is_unique
@@ -171,6 +212,25 @@ class UniqueGrouper(Grouper):
         else:
             return self._factorize_unique()
 
+    def _factorize_given_labels(self, group: T_Group) -> EncodedGroups:
+        codes = apply_ufunc(
+            _factorize_given_labels,
+            group,
+            kwargs={"labels": self.labels},
+            dask="parallelized",
+            output_dtypes=[np.int64],
+            keep_attrs=True,
+        )
+        return EncodedGroups(
+            codes=codes,
+            full_index=pd.Index(self.labels),  # type: ignore[arg-type]
+            unique_coord=Variable(
+                dims=codes.name,
+                data=self.labels,
+                attrs=self.group.attrs,
+            ),
+        )
+
     def _factorize_unique(self) -> EncodedGroups:
         # look through group to find the unique values
         sort = not isinstance(self.group_as_index, pd.MultiIndex)
@@ -276,17 +336,23 @@ class BinGrouper(Grouper):
     include_lowest: bool = False
     duplicates: Literal["raise", "drop"] = "raise"
 
+    def reset(self) -> Self:
+        return type(self)(
+            bins=self.bins,
+            right=self.right,
+            labels=self.labels,
+            precision=self.precision,
+            include_lowest=self.include_lowest,
+            duplicates=self.duplicates,
+        )
+
     def __post_init__(self) -> None:
         if duck_array_ops.isnull(self.bins).all():
             raise ValueError("All bin edges are NaN.")
 
-    def factorize(self, group: T_Group) -> EncodedGroups:
-        from xarray.core.dataarray import DataArray
-
-        data = np.asarray(group.data)  # Cast _DummyGroup data to array
-
-        binned, self.bins = pd.cut(  # type: ignore [call-overload]
-            data.ravel(),
+    def _cut(self, data):
+        return pd.cut(
+            np.asarray(data).ravel(),
             bins=self.bins,
             right=self.right,
             labels=self.labels,
@@ -296,23 +362,43 @@ class BinGrouper(Grouper):
             retbins=True,
         )
 
-        binned_codes = binned.codes
-        if (binned_codes == -1).all():
+    def _factorize_lazy(self, group: T_Group) -> DataArray:
+        def _wrapper(data, **kwargs):
+            binned, bins = self._cut(data)
+            if isinstance(self.bins, int):
+                # we are running eagerly, update self.bins with actual edges instead
+                self.bins = bins
+            return binned.codes.reshape(data.shape)
+
+        return apply_ufunc(_wrapper, group, dask="parallelized", keep_attrs=True)
+
+    def factorize(self, group: T_Group) -> EncodedGroups:
+        if isinstance(group, _DummyGroup):
+            group = DataArray(group.data, dims=group.dims, name=group.name)
+        by_is_chunked = is_chunked_array(group.data)
+        if isinstance(self.bins, int) and by_is_chunked:
+            raise ValueError(
+                f"Bin edges must be provided when grouping by chunked arrays. Received {self.bins=!r} instead"
+            )
+        codes = self._factorize_lazy(group)
+        if not by_is_chunked and (codes == -1).all():
             raise ValueError(
                 f"None of the data falls within bins with edges {self.bins!r}"
             )
 
         new_dim_name = f"{group.name}_bins"
+        codes.name = new_dim_name
 
-        full_index = binned.categories
-        uniques = np.sort(pd.unique(binned_codes))
-        unique_values = full_index[uniques[uniques != -1]]
+        # This seems silly, but it lets us have Pandas handle the complexity
+        # of `labels`, `precision`, and `include_lowest`, even when group is a chunked array
+        dummy, _ = self._cut(np.array([0]).astype(group.dtype))
+        full_index = dummy.categories
+        if not by_is_chunked:
+            uniques = np.sort(pd.unique(codes.data.ravel()))
+            unique_values = full_index[uniques[uniques != -1]]
+        else:
+            unique_values = full_index
 
-        codes = DataArray(
-            binned_codes.reshape(group.shape),
-            getattr(group, "coords", None),
-            name=new_dim_name,
-        )
         unique_coord = Variable(
             dims=new_dim_name, data=unique_values, attrs=group.attrs
         )
@@ -362,6 +448,15 @@ class TimeResampler(Resampler):
     index_grouper: CFTimeGrouper | pd.Grouper = field(init=False, repr=False)
     group_as_index: pd.Index = field(init=False, repr=False)
 
+    def reset(self) -> Self:
+        return type(self)(
+            freq=self.freq,
+            closed=self.closed,
+            label=self.label,
+            origin=self.origin,
+            offset=self.offset,
+        )
+
     def _init_properties(self, group: T_Group) -> None:
         from xarray import CFTimeIndex
 
@@ -432,8 +527,7 @@ class TimeResampler(Resampler):
         full_index, first_items, codes_ = self._get_index_and_items()
         sbins = first_items.values.astype(np.int64)
         group_indices: GroupIndices = tuple(
-            [slice(i, j) for i, j in zip(sbins[:-1], sbins[1:], strict=True)]
-            + [slice(sbins[-1], None)]
+            [slice(i, j) for i, j in pairwise(sbins)] + [slice(sbins[-1], None)]
         )
 
         unique_coord = Variable(
@@ -450,6 +544,21 @@ class TimeResampler(Resampler):
         )
 
 
+def _factorize_given_labels(data: np.ndarray, labels: np.ndarray) -> np.ndarray:
+    # Copied from flox
+    sorter = np.argsort(labels)
+    is_sorted = (sorter == np.arange(sorter.size)).all()
+    codes = np.searchsorted(labels, data, sorter=sorter)
+    mask = ~np.isin(data, labels) | isnull(data) | (codes == len(labels))
+    # codes is the index in to the sorted array.
+    # if we didn't want sorting, unsort it back
+    if not is_sorted:
+        codes[codes == len(labels)] = -1
+        codes = sorter[(codes,)]
+    codes[mask] = -1
+    return codes
+
+
 def unique_value_groups(
     ar, sort: bool = True
 ) -> tuple[np.ndarray | pd.Index, np.ndarray]:
