@@ -107,6 +107,10 @@ with warnings.catch_warnings():
     has_h5netcdf, requires_h5netcdf = _importorskip("h5netcdf")
 has_cftime, requires_cftime = _importorskip("cftime")
 has_dask, requires_dask = _importorskip("dask")
+has_dask_ge_2024_08_1, requires_dask_ge_2024_08_1 = _importorskip(
+    "dask", minversion="2024.08.1"
+)
+has_dask_ge_2024_11_0, requires_dask_ge_2024_11_0 = _importorskip("dask", "2024.11.0")
 with warnings.catch_warnings():
     warnings.filterwarnings(
         "ignore",
@@ -121,7 +125,7 @@ has_zarr, requires_zarr = _importorskip("zarr")
 has_zarr_v3, requires_zarr_v3 = _importorskip("zarr", "2.99")
 has_fsspec, requires_fsspec = _importorskip("fsspec")
 has_iris, requires_iris = _importorskip("iris")
-has_numbagg, requires_numbagg = _importorskip("numbagg", "0.4.0")
+has_numbagg, requires_numbagg = _importorskip("numbagg")
 has_pyarrow, requires_pyarrow = _importorskip("pyarrow")
 with warnings.catch_warnings():
     warnings.filterwarnings(
@@ -137,6 +141,7 @@ has_cartopy, requires_cartopy = _importorskip("cartopy")
 has_pint, requires_pint = _importorskip("pint")
 has_numexpr, requires_numexpr = _importorskip("numexpr")
 has_flox, requires_flox = _importorskip("flox")
+has_netcdf, requires_netcdf = _importorskip("netcdf")
 has_pandas_ge_2_2, requires_pandas_ge_2_2 = _importorskip("pandas", "2.2")
 has_pandas_3, requires_pandas_3 = _importorskip("pandas", "3.0.0.dev0")
 
@@ -151,39 +156,28 @@ requires_numbagg_or_bottleneck = pytest.mark.skipif(
     not has_numbagg_or_bottleneck, reason="requires numbagg or bottleneck"
 )
 has_numpy_2, requires_numpy_2 = _importorskip("numpy", "2.0.0")
-_, requires_flox_0_9_12 = _importorskip("flox", "0.9.12")
+has_flox_0_9_12, requires_flox_0_9_12 = _importorskip("flox", "0.9.12")
 
 has_array_api_strict, requires_array_api_strict = _importorskip("array_api_strict")
 
 
-def _importorskip_h5netcdf_ros3():
-    try:
-        import h5netcdf
-
-        has_h5netcdf = True
-    except ImportError:
-        has_h5netcdf = False
-
+def _importorskip_h5netcdf_ros3(has_h5netcdf: bool):
     if not has_h5netcdf:
         return has_h5netcdf, pytest.mark.skipif(
             not has_h5netcdf, reason="requires h5netcdf"
         )
 
-    h5netcdf_with_ros3 = Version(h5netcdf.__version__) >= Version("1.3.0")
-
     import h5py
 
     h5py_with_ros3 = h5py.get_config().ros3
 
-    has_h5netcdf_ros3 = h5netcdf_with_ros3 and h5py_with_ros3
-
-    return has_h5netcdf_ros3, pytest.mark.skipif(
-        not has_h5netcdf_ros3,
+    return h5py_with_ros3, pytest.mark.skipif(
+        not h5py_with_ros3,
         reason="requires h5netcdf>=1.3.0 and h5py with ros3 support",
     )
 
 
-has_h5netcdf_ros3, requires_h5netcdf_ros3 = _importorskip_h5netcdf_ros3()
+has_h5netcdf_ros3, requires_h5netcdf_ros3 = _importorskip_h5netcdf_ros3(has_h5netcdf)
 has_netCDF4_1_6_2_or_above, requires_netCDF4_1_6_2_or_above = _importorskip(
     "netCDF4", "1.6.2"
 )
@@ -216,8 +210,7 @@ class CountingScheduler:
         self.total_computes += 1
         if self.total_computes > self.max_computes:
             raise RuntimeError(
-                "Too many computes. Total: %d > max: %d."
-                % (self.total_computes, self.max_computes)
+                f"Too many computes. Total: {self.total_computes} > max: {self.max_computes}."
             )
         return dask.get(dsk, keys, **kwargs)
 
@@ -305,12 +298,12 @@ _DEFAULT_TEST_DIM_SIZES = (8, 9, 10)
 
 
 def create_test_data(
-    seed: int | None = None,
+    seed: int = 12345,
     add_attrs: bool = True,
     dim_sizes: tuple[int, int, int] = _DEFAULT_TEST_DIM_SIZES,
     use_extension_array: bool = False,
 ) -> Dataset:
-    rs = np.random.RandomState(seed)
+    rs = np.random.default_rng(seed)
     _vars = {
         "var1": ["dim1", "dim2"],
         "var2": ["dim1", "dim2"],
@@ -336,7 +329,7 @@ def create_test_data(
             "dim1",
             pd.Categorical(
                 rs.choice(
-                    list(string.ascii_lowercase[: rs.randint(1, 5)]),
+                    list(string.ascii_lowercase[: rs.integers(1, 5)]),
                     size=dim_sizes[0],
                 )
             ),
@@ -344,7 +337,7 @@ def create_test_data(
     if dim_sizes == _DEFAULT_TEST_DIM_SIZES:
         numbers_values = np.array([0, 1, 2, 0, 0, 1, 1, 2, 2, 3], dtype="int64")
     else:
-        numbers_values = rs.randint(0, 3, _dims["dim3"], dtype="int64")
+        numbers_values = rs.integers(0, 3, _dims["dim3"], dtype="int64")
     obj.coords["numbers"] = ("dim3", numbers_values)
     obj.encoding = {"foo": "bar"}
     assert_writeable(obj)
