@@ -10,7 +10,7 @@ from packaging.version import Version
 
 import xarray as xr
 from xarray.coding.cftime_offsets import _new_to_legacy_freq
-from xarray.core.pdcompat import _convert_base_to_offset
+from xarray.coding.cftimeindex import CFTimeIndex
 from xarray.core.resample_cftime import CFTimeGrouper
 
 cftime = pytest.importorskip("cftime")
@@ -60,10 +60,8 @@ def compare_against_pandas(
     freq,
     closed=None,
     label=None,
-    base=None,
     offset=None,
     origin=None,
-    loffset=None,
 ) -> None:
     if isinstance(origin, tuple):
         origin_pandas = pd.Timestamp(datetime.datetime(*origin))
@@ -77,8 +75,6 @@ def compare_against_pandas(
             time=freq,
             closed=closed,
             label=label,
-            base=base,
-            loffset=loffset,
             offset=offset,
             origin=origin_pandas,
         ).mean()
@@ -88,8 +84,6 @@ def compare_against_pandas(
                 time=freq,
                 closed=closed,
                 label=label,
-                base=base,
-                loffset=loffset,
                 origin=origin_cftime,
                 offset=offset,
             ).mean()
@@ -98,8 +92,6 @@ def compare_against_pandas(
             time=freq,
             closed=closed,
             label=label,
-            base=base,
-            loffset=loffset,
             origin=origin_cftime,
             offset=offset,
         ).mean()
@@ -116,14 +108,11 @@ def da(index) -> xr.DataArray:
     )
 
 
-@pytest.mark.filterwarnings("ignore:.*the `(base|loffset)` parameter to resample")
 @pytest.mark.parametrize("freqs", FREQS, ids=lambda x: "{}->{}".format(*x))
 @pytest.mark.parametrize("closed", [None, "left", "right"])
 @pytest.mark.parametrize("label", [None, "left", "right"])
-@pytest.mark.parametrize(
-    ("base", "offset"), [(24, None), (31, None), (None, "5s")], ids=lambda x: f"{x}"
-)
-def test_resample(freqs, closed, label, base, offset) -> None:
+@pytest.mark.parametrize("offset", [None, "5s"], ids=lambda x: f"{x}")
+def test_resample(freqs, closed, label, offset) -> None:
     initial_freq, resample_freq = freqs
     if (
         resample_freq == "4001D"
@@ -136,7 +125,6 @@ def test_resample(freqs, closed, label, base, offset) -> None:
             "result as pandas for earlier pandas versions."
         )
     start = "2000-01-01T12:07:01"
-    loffset = "12h"
     origin = "start"
 
     datetime_index = pd.date_range(
@@ -146,18 +134,15 @@ def test_resample(freqs, closed, label, base, offset) -> None:
     da_datetimeindex = da(datetime_index)
     da_cftimeindex = da(cftime_index)
 
-    with pytest.warns(FutureWarning, match="`loffset` parameter"):
-        compare_against_pandas(
-            da_datetimeindex,
-            da_cftimeindex,
-            resample_freq,
-            closed=closed,
-            label=label,
-            base=base,
-            offset=offset,
-            origin=origin,
-            loffset=loffset,
-        )
+    compare_against_pandas(
+        da_datetimeindex,
+        da_cftimeindex,
+        resample_freq,
+        closed=closed,
+        label=label,
+        offset=offset,
+        origin=origin,
+    )
 
 
 @pytest.mark.parametrize(
@@ -181,30 +166,22 @@ def test_closed_label_defaults(freq, expected) -> None:
 
 
 @pytest.mark.filterwarnings("ignore:Converting a CFTimeIndex")
-@pytest.mark.filterwarnings("ignore:.*the `(base|loffset)` parameter to resample")
 @pytest.mark.parametrize(
     "calendar", ["gregorian", "noleap", "all_leap", "360_day", "julian"]
 )
 def test_calendars(calendar: str) -> None:
     # Limited testing for non-standard calendars
-    freq, closed, label, base = "8001min", None, None, 17
-    loffset = datetime.timedelta(hours=12)
+    freq, closed, label = "8001min", None, None
     xr_index = xr.cftime_range(
         start="2004-01-01T12:07:01", periods=7, freq="3D", calendar=calendar
     )
     pd_index = pd.date_range(start="2004-01-01T12:07:01", periods=7, freq="3D")
-    da_cftime = (
-        da(xr_index)
-        .resample(time=freq, closed=closed, label=label, base=base, loffset=loffset)
-        .mean()
-    )
-    da_datetime = (
-        da(pd_index)
-        .resample(time=freq, closed=closed, label=label, base=base, loffset=loffset)
-        .mean()
-    )
+    da_cftime = da(xr_index).resample(time=freq, closed=closed, label=label).mean()
+    da_datetime = da(pd_index).resample(time=freq, closed=closed, label=label).mean()
     # TODO (benbovy - flexible indexes): update when CFTimeIndex is a xarray Index subclass
-    da_cftime["time"] = da_cftime.xindexes["time"].to_pandas_index().to_datetimeindex()
+    new_pd_index = da_cftime.xindexes["time"].to_pandas_index()
+    assert isinstance(new_pd_index, CFTimeIndex)  # shouldn't that be a pd.Index?
+    da_cftime["time"] = new_pd_index.to_datetimeindex()
     xr.testing.assert_identical(da_cftime, da_datetime)
 
 
@@ -214,7 +191,6 @@ class DateRangeKwargs(TypedDict):
     freq: str
 
 
-@pytest.mark.filterwarnings("ignore:.*the `(base|loffset)` parameter to resample")
 @pytest.mark.parametrize("closed", ["left", "right"])
 @pytest.mark.parametrize(
     "origin",
@@ -239,20 +215,12 @@ def test_origin(closed, origin) -> None:
     )
 
 
-@pytest.mark.filterwarnings("ignore:.*the `(base|loffset)` parameter to resample")
-def test_base_and_offset_error():
-    cftime_index = xr.cftime_range("2000", periods=5)
-    da_cftime = da(cftime_index)
-    with pytest.raises(ValueError, match="base and offset cannot"):
-        da_cftime.resample(time="2D", base=3, offset="5s")
-
-
 @pytest.mark.parametrize("offset", ["foo", "5MS", 10])
-def test_invalid_offset_error(offset) -> None:
+def test_invalid_offset_error(offset: str | int) -> None:
     cftime_index = xr.cftime_range("2000", periods=5)
     da_cftime = da(cftime_index)
     with pytest.raises(ValueError, match="offset must be"):
-        da_cftime.resample(time="2D", offset=offset)
+        da_cftime.resample(time="2D", offset=offset)  # type: ignore[arg-type]
 
 
 def test_timedelta_offset() -> None:
@@ -265,44 +233,3 @@ def test_timedelta_offset() -> None:
     timedelta_result = da_cftime.resample(time="2D", offset=timedelta).mean()
     string_result = da_cftime.resample(time="2D", offset=string).mean()
     xr.testing.assert_identical(timedelta_result, string_result)
-
-
-@pytest.mark.parametrize("loffset", ["MS", "12h", datetime.timedelta(hours=-12)])
-def test_resample_loffset_cftimeindex(loffset) -> None:
-    datetimeindex = pd.date_range("2000-01-01", freq="6h", periods=10)
-    da_datetimeindex = xr.DataArray(np.arange(10), [("time", datetimeindex)])
-
-    cftimeindex = xr.cftime_range("2000-01-01", freq="6h", periods=10)
-    da_cftimeindex = xr.DataArray(np.arange(10), [("time", cftimeindex)])
-
-    with pytest.warns(FutureWarning, match="`loffset` parameter"):
-        result = da_cftimeindex.resample(time="24h", loffset=loffset).mean()
-        expected = da_datetimeindex.resample(time="24h", loffset=loffset).mean()
-
-    result["time"] = result.xindexes["time"].to_pandas_index().to_datetimeindex()
-    xr.testing.assert_identical(result, expected)
-
-
-@pytest.mark.filterwarnings("ignore:.*the `(base|loffset)` parameter to resample")
-def test_resample_invalid_loffset_cftimeindex() -> None:
-    times = xr.cftime_range("2000-01-01", freq="6h", periods=10)
-    da = xr.DataArray(np.arange(10), [("time", times)])
-
-    with pytest.raises(ValueError):
-        da.resample(time="24h", loffset=1)  # type: ignore
-
-
-@pytest.mark.parametrize(("base", "freq"), [(1, "10s"), (17, "3h"), (15, "5us")])
-def test__convert_base_to_offset(base, freq):
-    # Verify that the cftime_offset adapted version of _convert_base_to_offset
-    # produces the same result as the pandas version.
-    datetimeindex = pd.date_range("2000", periods=2)
-    cftimeindex = xr.cftime_range("2000", periods=2)
-    pandas_result = _convert_base_to_offset(base, freq, datetimeindex)
-    cftime_result = _convert_base_to_offset(base, freq, cftimeindex)
-    assert pandas_result.to_pytimedelta() == cftime_result
-
-
-def test__convert_base_to_offset_invalid_index():
-    with pytest.raises(ValueError, match="Can only resample"):
-        _convert_base_to_offset(1, "12h", pd.Index([0]))
