@@ -14,6 +14,7 @@ from xarray.tests import (
     assert_identical,
     has_dask,
     requires_dask,
+    requires_dask_ge_2024_11_0,
     requires_numbagg,
 )
 
@@ -107,12 +108,13 @@ class TestDataArrayRolling:
         ):
             da.rolling(foo=2)
 
+    @requires_dask
     @pytest.mark.parametrize(
         "name", ("sum", "mean", "std", "min", "max", "median", "argmin", "argmax")
     )
     @pytest.mark.parametrize("center", (True, False, None))
     @pytest.mark.parametrize("min_periods", (1, None))
-    @pytest.mark.parametrize("backend", ["numpy"], indirect=True)
+    @pytest.mark.parametrize("backend", ["numpy", "dask"], indirect=True)
     def test_rolling_wrapped_bottleneck(
         self, da, name, center, min_periods, compute_backend
     ) -> None:
@@ -206,9 +208,9 @@ class TestDataArrayRolling:
             index=window, center=center, min_periods=min_periods
         ).reduce(np.nanmean)
 
-        np.testing.assert_allclose(s_rolling.values, da_rolling.values)
+        np.testing.assert_allclose(np.asarray(s_rolling.values), da_rolling.values)
         np.testing.assert_allclose(s_rolling.index, da_rolling["index"])
-        np.testing.assert_allclose(s_rolling.values, da_rolling_np.values)
+        np.testing.assert_allclose(np.asarray(s_rolling.values), da_rolling_np.values)
         np.testing.assert_allclose(s_rolling.index, da_rolling_np["index"])
 
     @pytest.mark.parametrize("center", (True, False))
@@ -221,12 +223,14 @@ class TestDataArrayRolling:
         da_rolling = da.rolling(index=window, center=center, min_periods=1)
 
         da_rolling_mean = da_rolling.construct("window").mean("window")
-        np.testing.assert_allclose(s_rolling.values, da_rolling_mean.values)
+        np.testing.assert_allclose(np.asarray(s_rolling.values), da_rolling_mean.values)
         np.testing.assert_allclose(s_rolling.index, da_rolling_mean["index"])
 
         # with stride
         da_rolling_mean = da_rolling.construct("window", stride=2).mean("window")
-        np.testing.assert_allclose(s_rolling.values[::2], da_rolling_mean.values)
+        np.testing.assert_allclose(
+            np.asarray(s_rolling.values[::2]), da_rolling_mean.values
+        )
         np.testing.assert_allclose(s_rolling.index[::2], da_rolling_mean["index"])
 
         # with fill_value
@@ -254,7 +258,7 @@ class TestDataArrayRolling:
         rolling_obj = da.rolling(time=window, center=center, min_periods=min_periods)
 
         # add nan prefix to numpy methods to get similar # behavior as bottleneck
-        actual = rolling_obj.reduce(getattr(np, "nan%s" % name))
+        actual = rolling_obj.reduce(getattr(np, f"nan{name}"))
         expected = getattr(rolling_obj, name)()
         assert_allclose(actual, expected)
         assert actual.sizes == expected.sizes
@@ -276,7 +280,7 @@ class TestDataArrayRolling:
         rolling_obj = da.rolling(time=window, center=center, min_periods=min_periods)
 
         # add nan prefix to numpy methods to get similar behavior as bottleneck
-        actual = rolling_obj.reduce(getattr(np, "nan%s" % name))
+        actual = rolling_obj.reduce(getattr(np, f"nan{name}"))
         expected = getattr(rolling_obj, name)()
         assert_allclose(actual, expected)
         assert actual.sizes == expected.sizes
@@ -310,7 +314,7 @@ class TestDataArrayRolling:
             DataArray([np.nan, np.nan, 2, 3, 3, 4, 5, 5, 5, 5, 5], dims="time"),
         ]
 
-        for kwarg, expected in zip(kwargs, expecteds):
+        for kwarg, expected in zip(kwargs, expecteds, strict=True):
             result = da.rolling(**kwarg).count()
             assert_equal(result, expected)
 
@@ -596,6 +600,42 @@ class TestDatasetRolling:
         ):
             ds.rolling(foo=2)
 
+    @requires_dask_ge_2024_11_0
+    def test_rolling_construct_automatic_rechunk(self):
+        import dask
+
+        # Construct dataset with chunk size of (400, 400, 1) or 1.22 MiB
+        da = DataArray(
+            dims=["latitute", "longitude", "time"],
+            data=dask.array.random.random((400, 400, 400), chunks=(-1, -1, 1)),
+            name="foo",
+        )
+
+        for obj in [da, da.to_dataset()]:
+            # Dataset now has chunks of size (400, 400, 100 100) or 11.92 GiB
+            rechunked = obj.rolling(time=100, center=True).construct(
+                "window",
+                sliding_window_view_kwargs=dict(
+                    automatic_rechunk=True, writeable=False
+                ),
+            )
+            not_rechunked = obj.rolling(time=100, center=True).construct(
+                "window",
+                sliding_window_view_kwargs=dict(
+                    automatic_rechunk=False, writeable=True
+                ),
+            )
+            assert rechunked.chunksizes != not_rechunked.chunksizes
+
+            roller = obj.isel(time=slice(30)).rolling(time=10, center=True)
+            one = roller.reduce(
+                np.sum, sliding_window_view_kwargs=dict(automatic_rechunk=True)
+            )
+            two = roller.reduce(
+                np.sum, sliding_window_view_kwargs=dict(automatic_rechunk=False)
+            )
+            assert_identical(one, two)
+
     @pytest.mark.parametrize(
         "name", ("sum", "mean", "std", "var", "min", "max", "median")
     )
@@ -649,7 +689,9 @@ class TestDatasetRolling:
             index=window, center=center, min_periods=min_periods
         ).mean()
 
-        np.testing.assert_allclose(df_rolling["x"].values, ds_rolling["x"].values)
+        np.testing.assert_allclose(
+            np.asarray(df_rolling["x"].values), ds_rolling["x"].values
+        )
         np.testing.assert_allclose(df_rolling.index, ds_rolling["index"])
 
     @pytest.mark.parametrize("center", (True, False))
@@ -668,7 +710,9 @@ class TestDatasetRolling:
         ds_rolling = ds.rolling(index=window, center=center)
 
         ds_rolling_mean = ds_rolling.construct("window").mean("window")
-        np.testing.assert_allclose(df_rolling["x"].values, ds_rolling_mean["x"].values)
+        np.testing.assert_allclose(
+            np.asarray(df_rolling["x"].values), ds_rolling_mean["x"].values
+        )
         np.testing.assert_allclose(df_rolling.index, ds_rolling_mean["index"])
 
         # with fill_value
@@ -695,7 +739,7 @@ class TestDatasetRolling:
         ds_rolling = ds.rolling(index=window, center=center)
         ds_rolling_mean = ds_rolling.construct("w", stride=2).mean("w")
         np.testing.assert_allclose(
-            df_rolling_mean["x"][::2].values, ds_rolling_mean["x"].values
+            np.asarray(df_rolling_mean["x"][::2].values), ds_rolling_mean["x"].values
         )
         np.testing.assert_allclose(df_rolling_mean.index[::2], ds_rolling_mean["index"])
 
@@ -704,7 +748,7 @@ class TestDatasetRolling:
         ds2_rolling = ds2.rolling(index=window, center=center)
         ds2_rolling_mean = ds2_rolling.construct("w", stride=2).mean("w")
         np.testing.assert_allclose(
-            df_rolling_mean["x"][::2].values, ds2_rolling_mean["x"].values
+            np.asarray(df_rolling_mean["x"][::2].values), ds2_rolling_mean["x"].values
         )
 
         # Mixed coordinates, indexes and 2D coordinates
@@ -741,7 +785,7 @@ class TestDatasetRolling:
         rolling_obj = ds.rolling(time=window, center=center, min_periods=min_periods)
 
         # add nan prefix to numpy methods to get similar behavior as bottleneck
-        actual = rolling_obj.reduce(getattr(np, "nan%s" % name))
+        actual = rolling_obj.reduce(getattr(np, f"nan{name}"))
         expected = getattr(rolling_obj, name)()
         assert_allclose(actual, expected)
         assert ds.sizes == actual.sizes
